################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
graph_reweight_links.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python graph_reweight_links.py --help

Type::

   python graph_reweight_links.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import tempfile
import time
import popen2

USAGE="""python %s [OPTIONS] < graph.in > graph.out

Version: $Id: graph_reweight_links.py 2782 2009-09-10 11:40:29Z andreas $

Wrapper for rescoring blast alignments.

Options:
-h, --help                      print this message.
-v, --verbose=                  loglevel.
-m, --method=                   method to use for scoring [bitscore|normalize]
-f, --self-score=               filename with self-scores
--lambda=                       lambda for bitscore calculation
--k=                            K for bitscore calculation
--expected=                     expected score in matrix

-d, --distance=                 convert to distance (# = maximum distance)

new scores are saved in third column (overwriting the E-Value)

Methods:

kimura                          = set third column to kimura two parameter score from pid
bitscore                        = set third column to bitscore from score.
normalize-product               = normalize third column by dividing with self-scores
                                new = old * old / self1 / self2
normalize-max                   = normalize third column by max(old/self1), max(old/self2)
normalize-min                   = normalize third column by min(old/self1), min(old/self2)
normalize-avg                   = normalize third column by avg((old/self1 + old/self2) / 2)

Format:

blast                           = Blast graph
edges                           = edge list (weighted)

""" % sys.argv[0]

param_long_options = ["help", "verbose=", "method=", "lambda=", "k=", "self-scores=", "expected=", "distance=",
                      "version"]

param_short_options = "hv:m:f:o:d:"

param_loglevel = 1
param_method = "bitscore"
param_filename_self_scores = None
param_format = "blast"

import CGAT.Experiment as E
import CGAT.BlastAlignments as BlastAlignments
import math

param_lambda = 0.267
param_K = 0.0410
param_expected = -0.5209
param_gop = -11.0
param_gep = -1.0
param_max_distance = 0.0

##-------------------------------------------------------------------------------
if __name__ == "__main__":

    try:
        optlist, args = getopt.getopt(sys.argv[1:], param_short_options, param_long_options)
    except getopt.error, msg:
        print USAGE, msg
        sys.exit(2)

    for o,a in optlist:
        if o in ( "-v", "--verbose" ):
            param_loglevel = int(a)
        elif o in ( "--version", ):
            print "version="
            sys.exit(0)
        elif o in ( "-h", "--help" ):
            print USAGE
            sys.exit(0)
        elif o in ("-m", "--method"):
            param_method = a
        elif o in ("-f", "--self-scores"):
            param_filename_self_scores = a
        elif o == "--lambda":
            param_lambda = float(a)
        elif o == "--k":
            param_K = float(a)
        elif o == "--expected":
            param_expected = float(a)
        elif o in ("-d", "--distance"):
            param_max_distance = float(a)
            
    print E.GetHeader()
    print E.GetParams()

    if param_filename_self_scores:
        self_scores = {}
        for line in open(param_filename_self_scores, "r"):
            if line[0] == "#": continue
            d = line[:-1].split("\t")[:2]
            if d[0] not in self_scores: self_scores[d[0]] = 0.0
            self_scores[d[0]] = max( self_scores[d[0]], float(d[1]))

    if param_method == "kimura":
        a = 0
        f = lambda x: x < 0.85 and 0.0000001 - math.log( 1.0 - x - 0.2 * x * x ) or 5.2030

    elif param_method == "bitscore":
        a = 1        
        lK = math.log(param_K)
        l2 = math.log(2)
        f = lambda x: (param_lambda * x - lK) / l2
        
    elif param_method in ("normalize-product", "normalize-max", "normalize-min", "normalize-avg"):
        a = 2
        if param_method == "normalize-product":
            f = lambda x,y,z: x * x / self_scores[y] / self_scores[z]
        elif param_method == "normalize-max":
            f = lambda x,y,z: max( x / self_scores[y], x / self_scores[z])
        elif param_method == "normalize-min":
            f = lambda x,y,z: min( x / self_scores[y], x / self_scores[z])
        elif param_method == "normalize-avg":
            f = lambda x,y,z: (x / self_scores[y] + x / self_scores[z]) / 2.0
    else:
        raise "unknown method %s" % param_method
    
    ninput, noutput, nskipped, nfailed = 0, 0, 0, 0

    for line in sys.stdin:

        if line[0] == "#": continue
        
        token1, token2, score = line.split("\t")[:3]
        try:
            score = float(score)
        except ValueError:
            # ignore headers
            continue
        ninput += 1
        
        if a == 0:
            score = f( (100.0 - score) / 100.0 )
        elif a == 1:
            score = f( score )
        elif a == 2:
            score = f( score, token1, token2 )

        
        if param_max_distance:
            score = param_max_distance - score

        print string.join(map( str, (token1, token2, score) ), "\t") 

        noutput += 1
            
    print "# ninput=%i, noutput=%i, nskipped=%i, failed=%i" % (\
        ninput, noutput, nskipped, nfailed )
    
    print E.GetFooter()





################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
Fasta.py - Methods for dealing with fasta files.
================================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python

"""

import string, os, sys, re

class Fasta:

    def __init__ (self,file):
        self.mFile = file
        self.mLastLine = " "
        
    def FetchOne( self ):
        """returns a tuple (description, sequence).

        Returns (None, None) if no more data is there.
        """
        while self.mLastLine != None:
            if self.mLastLine[0] == ">": break
            self.mLastLine = self.mFile.readline()
        else:
            return (None, None)
        
        description = self.mLastLine[1:-1]
        sequence = ""
        while 1:
            line = self.mFile.readline()
            if not line: break
            if line[0] == ">":
                self.mLastLine = line
                break

            sequence += line[:-1]

        return (description, re.sub("\s", "", sequence))
            
            
    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
data2roc.py - compute ROC data
==============================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script calculates a roc curve (true positive rate versus 
false positive rate)

Input is from stdin, first field is +/- or 1/0 for a true
or false positive, respectively. The input needs to be
sorted appropriately. 

If the option ``--false-negative`` is set, the input is +/- or 1/0 for a 
true positive or false negative, respectively.

Output on stdout is a tab-separated table with the following
columns:

TP: true positives
FP: false positives
TPR: true positive rate  = true_positives /  predicted
P: predicted
FPR: false positive rate = false positives  / predicted
value: value

Usage
-----

Example::

   python data2roc.py < data.in > roc.out

Type::

   python data2roc.py --help

for command line help.

Documentation
-------------

Code
----

'''

import string
import re
import getopt
import sys
import optparse

import CGAT.Experiment as E

def doMultiple( options ):

    first = True
    last_val = None
    headers, hists = [], []
    
    for line in sys.stdin:

        if line[0] == "#": continue

        data = line[:-1].split("\t")

        if first:
            headers = data[1:]
            ncols = len(headers)
            first = False
        else:
            val = data[0]
            if val != last_val:
                if last_val != None:
                    hists.append( (last_val, this_hist) )
                this_hist = [0] * ncols
                last_val = val

            for x in range(ncols):
                this_hist[x] += int(data[x+1])
                
    if last_val != None: hists.append( (last_val, this_hist) )

    if not headers: return
    options.stdout.write( "value\ttotal\t%s\t%s\n" % ("\t".join(headers),
                                                      "\t".join( ["p%s" % x for x in headers] ) ) )
    
    ntotal = 0
    hist = [0] * ncols
    for val, this_hist in hists:
        for x in range(ncols): hist[x] += this_hist[x]
        ntotal += sum( this_hist )
        options.stdout.write( "%s\t%i\t%s\t%s\n" % (val, ntotal,
                                                    "\t".join(["%i" % x for x in hist]),
                                                    "\t".join(["%f" % (float(x) / ntotal) for x in hist]) ) )


def main( argv = None ):

    if not argv: argv = sys.argv
 
    parser = E.OptionParser( version = "%prog version: $Id: data2roc.py 2782 2009-09-10 11:40:29Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-p", "--positives", dest="positives", type="float",
                      help="total number of true positives. If not set, take all true positives encountered [default=%default]." )

    parser.add_option("-m", "--monotonous", dest="monotonous", action="store_true",
                      help="bin data, so that sensitivity decreases monotonously [default=%default]." )

    parser.add_option("-b", "--bin", dest="bin", action="store_true",
                      help="bin data [default=%default]." )

    parser.add_option("-u", "--multiple", dest="multiple", action="store_true",
                      help="perform multiple flag analysis [default=%default]." )

    parser.add_option("-f", "--false-negatives", dest="false_negatives", action="store_true",
                      help="a negative flag indicates a false negative (and not a false positive) [default=%default]." )

    parser.set_defaults(
        positives = None,
        false_negatives = False,
        skip_redundant = True,
        monotonous = False,
        bin = False,
        multiple = False,
        bin_by_score = True )

    options, args = E.Start( parser, argv = sys.argv )

    if options.multiple:
        doMultiple( options )
        E.Stop()
        sys.exit(0)

    true_positives = 0
    predicted = 0

    values = []

    last_value = None

    ninput, noutput, nskipped = 0, 0, 0

    for line in sys.stdin:
        if line[0] == "#": continue
        
        data = string.split(line[:-1], "\t")

        ninput += 1
        if data[0] not in ("+", "-", "1", "0"): 
            nskipped += 1
            continue
        
        if not options.bin_by_score:
            if not options.bin or last_value != data[1]:
                values.append( (true_positives, predicted, data[1]) )
        else:
            if last_value != None and last_value != data[1]:
                values.append( (true_positives, predicted, last_value) )

        predicted += 1

        if data[0] in ("+", "1"):
            true_positives += 1

        last_value = data[1]

    values.append( (true_positives, predicted, last_value) )
    values.append( (true_positives, predicted, data[1]) )

    if true_positives == 0:
        raise ValueError("# no true positives!")
        
    if options.positives == None:
        if options.false_negatives:
            positives = float(predicted)
        else:
            positives = float(true_positives)
    else:
        positives = float(options.positives)

    options.stdout.write( "value\tpred\tTP\tFP\tTN\tFN\tTPR\tFPR\tTNR\tFNR\tRTPR\tRFNR\n" )

    last_positives = None
    last_tpr = None

    for true_positives, predicted, value in values:

        if (predicted == 0):
            predicted = 1

        if options.false_negatives:
            false_negatives = predicted - true_positives
            false_positives = 0
            true_negatives = 0
        else:
            true_negatives = 0
            false_negatives = positives - true_positives
            false_positives = predicted - true_positives

        tpr = float(true_positives) / predicted
        fpr = float(false_positives) / (true_positives + false_negatives )
        fnr = float(false_negatives) / positives
        tnr = 0

        # relative rates 
        rfpr = float(false_positives) / predicted
        rfnr = float(false_negatives) / predicted

        if options.monotonous and last_tpr and last_tpr < tpr:
            continue

        if options.skip_redundant and true_positives == last_positives:
            continue
        
        if (predicted > 0):
            options.stdout.write( "%s\t%i\t%i\t%i\t%i\t%i\t%f\t%f\t%f\t%f\t%f\t%f\n" %\
                                      (value,
                                       predicted,
                                       true_positives,
                                       false_positives,
                                       true_negatives,
                                       false_negatives,
                                       tpr, fpr, tnr, fnr,
                                       rfpr, rfnr ) )


            noutput ++ 1

        last_positives = true_positives
        last_tpr = tpr

    E.info( "ninput=%i, noutput=%i, nskipped=%i" % (ninput, noutput, nskipped))

    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv ) )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
solexa2stats.py - compute stats from solexa export files
========================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script returns the genomic insert size distances between mate-paired reads mapping to the same chromosome.  Hence, these values could in theory be negative if the fragment was so short that sequenced ends overlap.  It assumes it is piped an Illumina .export file::

   zcat whatever.export.gz | $RMAAPATH/map/return_insert_sizes > file.for.further.manipulation

The export file is assumed to be organized with mate pairs next to each other as follows:

#. Read 1234/1
#. Read 1234/2
#. Read 1635/1
#. Read 1635/2
#. etc.

Inner-distances are returned on individual lines as follows:

#. 101
#. 120
#. 115
#. etc.

Usage
-----

Example::

   python cgat_script_template.py --help

Type::

   python cgat_script_template.py --help

for command line help.

Documentation
-------------

Code
----

'''

import os
import sys
import re
import optparse
import collections

import CGAT.Experiment as E

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-i", "--test-option", dest="test_option", type="string",
                      help="test option [default=%default]."  )

    parser.set_defaults(
        test_option = "test"
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    infile = options.stdin
    outfile = options.stdout

    line=infile.readline()
    line2=infile.readline()
    seq_len=len(line.split("\t")[8])
   
    counts = collections.defaultdict( int )

    while 1:

       data = line.split("\t")
       data2 = line2.split("\t")

       try:
          if data[10]==data[10] and 'chr' in data[10]:
             size = abs(int(data[12])-int(data2[12]))-seq_len
             counts[size] += 1
       except (ValueError, IndexError):
          pass

       line=infile.readline()
       line2=infile.readline()

       if not line or not line2: break

    outfile.write( "size\tcounts\n" )
    for key in sorted(counts.keys()):
       outfile.write( "%i\t%i\n" % (key, counts[key]) )

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )





################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: snp2table.py 2861 2010-02-23 17:36:32Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
annotator_distance.py - statistical significance of distance between genomic segments
=====================================================================================

Purpose
-------

The script :file:`annotator_distance.py` computes the statististical significance 
of the association between segments on a genome.

The genome is arranged into ``workspaces``, contiguous blocks in the genome that are
labeled at their termini. An example of workspaces are intergenic spaces labeled ``3'`` 
or ``5'`` according to the adjacent gene.

Within the workspace are ``segments`` of interest, for example transcription factor binding
sites. The script counts the distance of a segment to the nearest ``workspace`` terminus. 
The counts are aggregated over all workspaces.

Next, the script randomly rearranges ``segments`` within a ``workspace`` in order to test the 
statistical significance of the observed counts.

The script implements various sampling and counting methods.

Usage
-----

Type::

   python annotator_distances.py --help

for command line usage.

Code
----
"""

import os
import sys
import optparse
import collections
import itertools
import CGAT.GTF as GTF
import CGAT.Bed as Bed
import CGAT.Intervals as Intervals
import CGAT.IOTools as IOTools
import CGAT.Experiment as E
import bx.intervals.io
import bx.intervals.intersection
import numpy
import random
import math
import array
import scipy
import scipy.stats

import matplotlib.pyplot as plt
from matplotlib import _pylab_helpers

## global functions, defined once for optimization purposes
normalize_transform = lambda x, y: numpy.array( x, float) / ( sum(x) + y)
cumulative_transform = lambda x, y: numpy.cumsum( numpy.array( x, float) / (sum(x) + y) )

def readWorkspace( infile, 
                   workspace_builder = "raw", 
                   label="none",
                   map_id2annotation = {} ):
    """read workspace from infile.

    A workspace is a collection of intervals with two labels associated
    to each interval, one for the 5' and one for the 3' end.

    Available workspace builders are:

    gff
       take a gff file. 

    gtf-intergenic
       build workspace from intergenic segments in a gtf file. 

    gtf-intronic
       build workspace from intronic segments in a gtf file

    gtf-genic
       the workspace is built from genes (first to last exon).

    Available labels are:

    none
       no labels are given to the ends of workspaces

    direction
       labels are given based on the 5'/3' end of the
       bounding exon

    annotation
       labels are given based on a gene2annotation map.

    returns a list of segments for each contig in a dictionary
    """

    if label == "none":
        label_f = lambda x,y: (("X",),("X",))
        info_f = lambda x: None
    elif label == "direction":
        label_f = lambda x,y: ( ( ("5","3")[x],), ( ("3","5")[y],) )
        info_f = lambda x: x.strand == "+"
    elif label == "annotation":
        label_f = lambda x,y: (map_id2annotation[x], map_id2annotation[y])
        info_f = lambda x: x.gene_id
        
    if workspace_builder == "gff":
        workspace = GTF.readAsIntervals( GFF.iterator( infile ) )

    elif workspace_builder == "gtf-intergenic":

        workspace = collections.defaultdict( list )
        # get all genes
        for e in GTF.merged_gene_iterator( GTF.iterator( infile ) ):
            workspace[e.contig].append( (e.start, e.end, info_f( e ) ) )

        # convert to intergenic regions.
        # overlapping genes are merged and the labels
        # of the right-most entry is retained
        for contig in workspace.keys():
            segs = workspace[contig]
            segs.sort()
            last = segs[0]
            new_segs = []
            for this in segs[1:]:
                if last[1] >= this[0]:
                    if this[1] > last[1]:
                        last = (last[0], this[1], this[2] )
                    continue
                assert last[1] < this[0], "this=%s, last=%s" % (this, last)

                new_segs.append( (last[1], this[0], 
                                  label_f( last[2], this[2] ) ) )
                last = this
            workspace[contig] = new_segs

    elif workspace_builder == "gtf-intronic":

        workspace = collections.defaultdict( list )
        
        # the current procedure will count nested genes
        # twice
        for ee in GTF.flat_gene_iterator( GTF.iterator( infile) ):
            
            exons = Intervals.combine( [ (e.start, e.end) for e in ee ] )
            introns = Intervals.complement( exons ) 

            r = ee[0]
            for start, end in introns:
                workspace[r.contig].append( (start, 
                                              end, 
                                              label_f( info_f( r ), info_f(r) ) 
                                              ) )
    elif workspace_builder == "gtf-genic":

        workspace = collections.defaultdict( list )
        
        # the current procedure will count nested genes
        # twice
        for ee in GTF.flat_gene_iterator( GTF.iterator( infile) ):

            exons = Intervals.combine( [ (e.start, e.end) for e in ee ] )
            start, end = exons[0][0], exons[-1][1]
            r = ee[0]
            workspace[r.contig].append( (start, 
                                          end, 
                                          label_f( info_f( r ), info_f(r) ) 
                                          ) )

    else:
        raise ValueError("unknown workspace_builder %s" % workspace_builder)

    return workspace

def readSegments( infile, indexed_workspace, 
                  truncate = False, 
                  format="gtf", 
                  keep_ambiguous = False,
                  remove_overhangs = False ):
    """read segments from infile.

    segments not overlapping with indexed_workspace are removed.

    If :attr: truncate is given, segments extending beyond the workspace
    are truncated.
    
    returns a list of segments for each contig in a dictionary
    """
    counter = E.Counter()

    segments = collections.defaultdict( list )

    def addSegment( contig, start, end, counter ):
        if contig in indexed_workspace:
            r = indexed_workspace[contig].find( start, end )
            if not r:
                counter.nskipped += 1
                return
            if len(r) > 1:
                counter.nambiguous += 1
                if not keep_ambiguous: 
                    return
            if truncate:
                for x in r:
                    wstart, wend = x.start, x.end
                    rstart, rend = max( start, wstart), min( end, wend)
                    if start < wstart or end > wend: counter.ntruncated += 1
                    segments[contig].append( (rstart, rend) )
                    counter.added += 1
            elif remove_overhangs:
                for x in r:
                    wstart, wend = x.start, x.end
                    rstart, rend = max( start, wstart), min( end, wend)
                    if start < wstart or end > wend: 
                        counter.overhangs +=1
                        break
                else:
                    segments[contig].append( (start, end) )
            else:
                segments[contig].append( (start, end) )
                counter.added += 1

            counter.nkept += 1
    
    if format == "gtf":
        gtf_iterator = GTF.flat_gene_iterator( GTF.iterator( infile ) )

        for gene in gtf_iterator:
            # get start and end ignoring introns
            # contig, start, end = gene[0].contig, min( [x.start for x in gene] ), max( [x.end for x in gene] )

            contig, coords = gene[0].contig, [(x.start, x.end) for x in gene ]
            counter.ninput += 1
            for start,end in coords:
                addSegment( contig, start, end, counter )

    elif format == "bed":
        bed_iterator = Bed.iterator( infile )
        for bed in bed_iterator:
            counter.ninput += 1
            addSegment( bed.contig, bed.start, bed.end, counter )

    E.info( "read segments: %s" % str(counter) )

    return segments

class Sampler(object):
    """base clase for objcects that create a sample of
    randomly arranged segments in a workspace.

    """

    def __init__(self, observed, work_start, work_end ):
        self.mObserved = observed

        self.mWorkStart, self.mWorkEnd = work_start, work_end

        self.mLengths = [x[1] - x[0] for x in observed ]
        self.mTotalLength = sum( self.mLengths )
        self.mFreeLength = work_end - work_start - self.mTotalLength

        assert self.mFreeLength >= 0, "negative length: workspace=(%i,%i) %i-%i<0, segments=%s, lengths=%s" % \
            (work_start, work_end, work_end-work_start, self.mTotalLength, self.mObserved, self.mLengths)

    def sample( self ):
        raise NotImplementedError( "define sample() in base classes" )
        
class SamplerPermutation(Sampler):
    """permute order of fragments and distribute randomly.

    The permutation works like this:
    1. Randomly permutate the order of segments
    2. Split the free space (:attr:mFreeSpace) within the workspace into n+1 randomly sized gaps
    3. Insert the gaps between permutated segments

    """
    def sample( self ):
        """return simulated fragments."""
        simulated = []
        # 1. permutate order of segments
        random.shuffle( self.mLengths )
        # 2. determine size of space between samples
        points = []
        for x in range(len(self.mLengths) + 1 ):
            points.append( random.randint( 0, self.mFreeLength ) )
        points.sort()
        # 3. move segments to appropriate place
        start = self.mWorkStart
        simulated = []
        last = 0
        for x in range(len(self.mLengths)):
            start += points[x] - last
            simulated.append( (start, start + self.mLengths[x]) )
            start += self.mLengths[x]
            last = points[x]

        assert start + (points[-1] - last) <= self.mWorkEnd, "start=%i, points[-1]=%i, work_end=%i" % \
            (start, points[-1] -last, self.mWorkEnd)

        return simulated

class SamplerBlocks(Sampler):
    """move blocks of fragments to take into account clustering."""
    def sample( self ):
        """return simulated fragments."""
        simulated = []
        raise NotImplementedError

class SamplerGaps(Sampler):
    """rearrange gaps within a block randomly. 

    This sampler will preserve same of the clustering structure of segments."""
    def __init__(self, *args, **kwargs):
        Sampler.__init__(self,*args, **kwargs)
        self.mGapLengths = [x[1] - x[0] for x in Intervals.complement( self.mObserved, self.mWorkStart, self.mWorkEnd )]

    def sample( self ):
        """return simulated fragments."""
        simulated = []
        gaps = self.mGapLengths
        random.shuffle( gaps )
        
        start = self.mWorkStart

        for x in range(len(self.mLengths)):
            start += gaps[x]
            simulated.append( (start, start + self.mLengths[x]) )
            start += self.mLengths[x]

        return simulated

class CountingResults(object):
    """a container for observed and simulated counts.
    """
    def __init__(self,labels):
        self.mLabels = labels

        self.mTransform = None
        self.mEnvelopes = {}
        self.mMedians = {}

        self.mObservedCounts = None
        self.mSimulatedCounts = None
        self.mStats = None

    def updateFDR( self, obs_pvalues, sim_pvalues ):
        """compute fdr stats with given counts.

        If obs_pvalues and sim_pvalues are given, computes the FDR (q-value) for the observed p-value.
        The q-value is the expected proportion of false positive observations at
        the observed p-value.

        qvalue = A / B
        A: average proportion of simulated data with P-Values < pvalue (expected false positive RATE)
        B: number of observed data with P-Values < pvalue (NUMBER of true positives)

        As there are several counters and labels, all observed and simulated pvalues
        are taken into account.

        The method needs to be called after :meth:update.
        """

        assert self.mStats != None, "updateFDR called before calling update."

        for label in self.mLabels:
            pvalue = self.mStats[label].pvalue
            a = scipy.stats.percentileofscore( sim_palues, pvalue ) / 100.0
            b = scipy.stats.percentileofscore( obs_pvalues, pvalue ) / 100.0 * len(obs_pvalues ) 
            if b >= 0:
                qvalue = min( 1.0, a / b )
            else:
                qvalue = 0

            self.mStats[label] = self.mStats[label]._replace( qvalue=qvalue)

    def update( self ):
        """update stats from given counts.
        """
        
        assert self.mObservedCounts != None, "update called without observed counts."
        assert self.mSimulatedCounts != None, "update called without simulated counts."

        self.mStats = {}
        
        cls = collections.namedtuple( "st", "observed expected ci95lower ci95upper pvalue qvalue" )

        for label in self.mLabels:
            obs = cumulative_transform(self.mObservedCounts[label], self.mObservedCounts.mOutOfBounds[label])
            pobs = findMedian( obs )
            medians = self.getMedians( label )
            medians.sort()
            pvalue = float(scipy.stats.percentileofscore( medians, pobs )) / 100.0
            self.mStats[label] = cls( 
                pobs,
                scipy.mean( medians ),
                scipy.stats.scoreatpercentile( medians, 5 ),
                scipy.stats.scoreatpercentile( medians, 95 ),
                pvalue, None )

    def getLabels( self ):
        return self.mLabels

    def getMedians( self, label ):
        """compute medians of all samples."""
        if label not in self.mMedians: 

            num_samples = len(self.mSimulatedCounts)

            medians = []

            for x in range(num_samples):
                data = self.mSimulatedCounts[x][label]
                threshold = self.mSimulatedCounts[x].mTotals[label] / 2
                t = 0
                for d in range(len(data)):
                    if t > threshold: break
                    t += data[d]
                medians.append( d )

            self.mMedians[label] = medians

        return self.mMedians[label]

    def getEnvelope( self, label, transform ):
        """compute envelope for label using transform.

        The envelope is the min, max and mean of the observed counts
        add a certain position.

        This function does a lazy evaluation. Pre-computed results are stored
        and returned if the same transform is applied.
        """
        if label in self.mEnvelopes and transform == self.mTransform:
            E.debug( "returning cached envelope for transform %s" % str(transform) )
            return self.mEnvelopes[label]

        E.debug( "computing new envelope for transform %s" % str(transform) )

        num_samples = len(self.mSimulatedCounts)

        mmin = numpy.array( transform( self.mSimulatedCounts[0][label], self.mSimulatedCounts[0].mOutOfBounds[label]), numpy.float )
        msum = numpy.array( transform( self.mSimulatedCounts[0][label], self.mSimulatedCounts[0].mOutOfBounds[label]), numpy.float )
        mmax = numpy.array( transform( self.mSimulatedCounts[0][label], self.mSimulatedCounts[0].mOutOfBounds[label]), numpy.float )

        for x in range(1,num_samples):
            v = transform( self.mSimulatedCounts[x][label], self.mSimulatedCounts[x].mOutOfBounds[label])
            mmin = numpy.minimum( mmin, v )
            mmax = numpy.maximum( mmax, v )
            msum = msum + v
                
        msum /= num_samples

        self.mTransform = transform

        self.mEnvelopes[label] = ( mmin, mmax, msum )

        return self.mEnvelopes[label]

class Counter( object ):
    """return object that counts segments in a workspace.

    A counter will implement an addCounts method that expects a sorted
    list of intervals within a region bounded by start,end.
    """

    ## python list is fastest for single value access, but requires a lot of
    ## memory. python array is a good compromise - slightly slower than python list
    ## but uses much less space. For range access, use numpy arrays.
    mBuildCounts = lambda self, num_bins, dtype: array.array( "I", [0] * num_bins )

    def __init__(self, labels, num_bins, resolution = 1, dtype = numpy.int8 ):

        self.mCounts = {}
        self.mTotals = {}
        # keep separate out-of-bounds counts in order to not interfere with
        # dtype
        self.mOutOfBounds = {}

        b = self.mBuildCounts
        for l in labels:
            self.mCounts[l] = self.mBuildCounts( num_bins, dtype )
            self.mTotals[l] = 0
            self.mOutOfBounds[l] = 0

        self.mNumBins = num_bins
        self.mResolution = resolution
        
    def __getitem__( self, key ):
        return self.mCounts[key]

    def getLabels( self ):
        return self.mCounts.keys()

    def resolve( self, value ):
        if self.mResolution > 1:
            return int(math.floor( float(value) / self.mResolution) )
        else:
            return value

class CounterTranscription( Counter ):
    """count transcription per base."""

    mName = "Transcription"

    ## numpy is fastest for counting with blocks of data
    mBuildCounts = lambda self, num_bins, dtype: numpy.zeros( num_bins, dtype )

    def addCounts( self, rr, start, end, left_labels, right_labels ):

        counts = self.mCounts
        totals = self.mTotals
        ofb = self.mOutOfBounds
        nbins = self.mNumBins
        resolve = self.resolve

        for istart, iend in rr:
            l = iend - istart
            dl = istart - start
            dr = end - iend
            l = self.resolve( l )
            if dl < dr:
                pos = self.resolve( dl )
                labels = left_labels
            elif dl > dr:
                pos = self.resolve( dr )
                labels = right_labels
            else:
                continue

            if pos >= nbins:
                for label in labels:
                    ofb[label] += l
                    totals[label] += l
            else:
                for label in labels:
                    counts[label][pos:pos+l] += 1
                    totals[label] += l

class CounterClosestDistance( Counter ):
    """count closest distance."""

    mName = "Closest distance"
    def addCounts( self, rr, start, end, left_labels, right_labels ):

        counts = self.mCounts
        totals = self.mTotals
        ofb = self.mOutOfBounds
        nbins = self.mNumBins
        resolve = self.resolve

        def __add( pos, labels ):
            if pos >= nbins:
                for label in labels:
                    self.mOutOfBounds[label] += 1
                    totals[label] += 1
            else:
                for label in labels:
                    counts[label][pos] += 1
                    totals[label] += 1

        pos = self.resolve( rr[0][0] - start )
        __add( pos, left_labels )
        pos = self.resolve( end - rr[-1][1] )
        __add( pos, right_labels )

class CounterAllDistances( Counter ):
    """count all distances."""

    mName = "All distances"
    def addCounts( self, rr, start, end, left_labels, right_labels ):

        counts = self.mCounts
        totals = self.mTotals
        ofb = self.mOutOfBounds
        nbins = self.mNumBins
        resolve = self.resolve

        for istart, iend in rr:
            dl = istart - start
            dr = end - iend
            if dl < dr:
                pos = resolve( dl )
                labels = left_labels
            elif dl > dr:
                pos = resolve( dr )
                labels = right_labels
            else: 
                continue

            if pos >= nbins:
                for label in labels:
                    ofb[label] += 1
                    totals[label] += 1
            else:
                for label in labels:
                    counts[label][pos] += 1
                    totals[label] += 1

def indexIntervals( intervals, with_values = False ):
    """index intervals using bx.
    """

    indexed = {}

    for contig, values in intervals.iteritems():
        intersector = bx.intervals.intersection.Intersecter()
        if with_values:
            for start, end, value in values:
                intersector.add_interval( bx.intervals.Interval(start,end,value=value) )
        else:
            for start, end in values:
                intersector.add_interval( bx.intervals.Interval(start,end) )
        indexed[contig] = intersector
    return indexed

def plotCounts( counter, options, transform = lambda x: x ):
    """create plots from counter."""

    num_bins = options.num_bins
    resolution = options.resolution
    bins = numpy.array( xrange(num_bins) ) * resolution
    for label in counter.getLabels():
        fig = plt.figure()

        if options.plot_samples:
            for x in range(options.num_samples):
                counts = transform( counter.mSimulatedCounts[x][label], counter.mSimulatedCounts[x].mOutOfBounds[label] )
                plt.plot( bins, counts / t, label = "sample_%i" % x )

        if options.plot_envelope:
            # counts per sample are in row
            mmin, mmax, mmean = counter.getEnvelope( label, transform )

            plt.plot( bins, mmin, label = "min" )
            plt.plot( bins, mmax, label = "max" )
            plt.plot( bins, mmean, label = "mean" )

        plt.plot( bins, transform( counter.mObservedCounts[label], counter.mObservedCounts.mOutOfBounds[label] ), label = "observed" ) 
        plt.xlim( options.xrange )
        plt.legend()
        plt.title( counter.mName )

        plt.xlabel( "distance from gene / bp" )
        plt.ylabel( "frequency" )

        fig.suptitle( str(label) )

        if options.logscale:
            if "x" in options.logscale:
                plt.gca().set_xscale('log')
            if "y" in options.logscale:
                plt.gca().set_yscale('log')

        if options.hardcopy:
            plt.savefig( os.path.expanduser(options.hardcopy % label) )   

def findMedian( dist ):
    """find median in cumulative and normalized distribution."""
    x = 0
    while dist[x] < 0.5: x+=1
    return x

def main( argv = sys.argv ):

    parser = E.OptionParser( version = "%prog version: $Id: annotator_distance.py 2861 2010-02-23 17:36:32Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-a", "--filename-annotations", dest="filename_annotations", type="string",
                      help="filename mapping gene ids to annotations (a tab-separated table with two-columns) [default=%default]." )

    parser.add_option("-r", "--resolution", dest="resolution", type="int",
                      help="resolution of count vector [default=%default]." )

    parser.add_option("-b", "--num-bins", dest="num_bins", type="int",
                      help="number of bins in count vector [default=%default]." )

    parser.add_option("-i", "--num-samples", dest="num_samples", type="int",
                      help="sample size to compute [default=%default]." )
    
    parser.add_option("-w", "--workspace", dest="filename_workspace", type="string",
                      help="filename with workspace information [default=%default]." )
    
    parser.add_option( "--workspace-builder", dest="workspace_builder", type="choice",
                       choices=("gff", "gtf-intergenic", "gtf-intronic", "gtf-genic" ),
                       help="given a gff/gtf file build a workspace [default=%default]." )

    parser.add_option( "--workspace-labels", dest="workspace_labels", type="choice",
                       choices=("none", "direction", "annotation"),
                       help="labels to use for the workspace workspace [default=%default]." )

    parser.add_option( "--sampler", dest="sampler", type="choice",
                       choices=("permutation", "gaps"),
                       help="sampler to use. The sampler determines the null model of how segments are distributed in the workspace  [default=%default]" )
    
    parser.add_option( "--counter", dest="counters", type="choice", action="append",
                       choices=("transcription", "closest-distance", "all-distances"),
                       help="counter to use. The counter computes the quantity of interest [default=%default]" )

    parser.add_option( "--analysis", dest="analysis", type="choice", action="append",
                       choices=("proximity", "area-under-curve"),
                       help="analysis to perform [default=%default]" )

    parser.add_option( "--transform-counts", dest="transform_counts", type="choice",
                       choices=("raw", "cumulative" ),
                       help="cumulate counts [default=%default]." )

    parser.add_option("-s", "--segments", dest="filename_segments", type="string",
                      help="filename with segment information [default=%default]." )

    parser.add_option( "--xrange", dest="xrange", type="string",
                       help="xrange to plot [default=%default]")

    parser.add_option("-o", "--logscale", dest="logscale", type="string",
                      help="use logscale on x, y or xy [default=%default]"  )

    parser.add_option("-p", "--plot", dest="plot", action="store_true",
                      help="output plots [default=%default]"  )

    parser.add_option(  "--hardcopy", dest="hardcopy", type="string",
                      help="output hardcopies to file [default=%default]"  )

    parser.add_option( "--no-fdr", dest="do_fdr", action="store_false",
                      help="do not compute FDR rates [default=%default]"  )

    parser.add_option( "--segments-format", dest="segments_format", type="choice",
                       choices=("gtf", "bed" ),
                       help="format of segments file [default=%default]." )

    parser.add_option( "--truncate", dest="truncate", action="store_true",
                       help="truncate segments extending beyond a workspace [default=%default]"  )

    parser.add_option( "--remove-overhangs", dest="remove_overhangs", action="store_true",
                       help="remove segments extending beyond a workspace[default=%default]"  )

    parser.add_option( "--keep-ambiguous", dest="keep_ambiguous", action="store_true",
                       help="keep segments extending to more than one workspace [default=%default]"  )
    
    parser.set_defaults(
        filename_annotations = None,
        filename_workspace = "workspace.gff",
        filename_segments = "FastDown.gtf",
        filename_annotations_gtf = "../data/tg1_territories.gff",
        workspace_builder = "gff",
        workspace_labels = "none",
        sampler = "permutation",
        truncate = False,
        num_bins = 10000,
        num_samples = 10,
        resolution = 100,
        plot_samples = False,
        plot_envelope = True,
        counters = [],
        transform_counts = "raw",
        xrange = None,
        plot = False,
        logscale = None,
        output_all = False,
        do_test = False,
        analysis = [],
        do_fdr = True,
        hardcopy = "%s.png",
        segments_format = "gtf",
        remove_overhangs = False,
        )

    (options, args) = E.Start( parser, argv = argv, add_output_options = True )

    ###########################################
    ## setup options
    if options.sampler == "permutation":
        sampler = SamplerPermutation
    elif options.sampler == "gaps":
        sampler = SamplerGaps

    if options.xrange: options.xrange = map(float, options.xrange.split(",") )

    if len(options.counters) == 0:
        raise ValueError("please specify at least one counter.")

    if len(options.analysis) == 0:
        raise ValueError("please specify at least one analysis.")

    if options.workspace_labels == "annotation" and not options.filename_annotations:
        raise ValueError("please specify --filename-annotations is --workspace-labels=annotations.")

    ###########################################
    ## read data
    if options.workspace_labels == "annotation":
        def constant_factory(value):
            return itertools.repeat(value).next

        def dicttype():
            return collections.defaultdict( constant_factory( ("unknown",) ) )

        map_id2annotations = IOTools.readMultiMap( open( options.filename_annotations, "r"),
                                                   dtype = dicttype )
    else:
        map_id2annotations = {}

    workspace = readWorkspace( open( options.filename_workspace, "r" ), 
                               options.workspace_builder,
                               options.workspace_labels, 
                               map_id2annotations )

    
    E.info( "read workspace for %i contigs" % (len(workspace)) )

    indexed_workspace = indexIntervals( workspace, with_values = True )
    segments = readSegments( open( options.filename_segments, "r" ), indexed_workspace, 
                             format = options.segments_format,
                             keep_ambiguous = options.keep_ambiguous,
                             truncate = options.truncate,
                             remove_overhangs = options.remove_overhangs )

    nsegments = 0
    for contig, vv in segments.iteritems():
        nsegments += len(vv)
    
    E.info( "read %i segments for %i contigs" % (nsegments, len(workspace)) )
    indexed_segments = indexIntervals( segments, with_values = False)

    if nsegments == 0:
        E.warn( "no segments read - no computation done.")
        E.Stop()
        return
    
    # build labels
    labels = collections.defaultdict( int )
    for contig, vv in workspace.iteritems():
        for start, end, v in vv:
            for l in v[0]: labels[l] += 1
            for l in v[1]: labels[l] += 1

    E.info( "found %i workspace labels" % len(labels ) )
        
    ###########################################
    ## setup counting containers
    counters = []
    for cc in options.counters:
        
        if cc == "transcription":
            counter = CounterTranscription
        elif cc == "closest-distance":
            counter = CounterClosestDistance
        elif cc == "all-distances":
            counter = CounterAllDistances
            
        if nsegments < 256: 
            dtype = numpy.uint8
        elif nsegments < 65536: 
            dtype = numpy.uint16
        elif nsegments < 4294967296:
            dtype = numpy.uint32
        else:
            dtype = numpy.int

        E.debug( "choosen dtype %s" % str(dtype))

        E.info( "samples space is %i bases: %i bins at %i resolution" % \
                    (options.num_bins * options.resolution,
                     options.num_bins, 
                     options.resolution,
                     ) )

        E.info( "allocating counts: %i bytes (%i labels, %i samples, %i bins)" %\
                ( options.num_bins * len(labels) * dtype().itemsize * (options.num_samples + 1),
                  len(labels), 
                  options.num_samples,
                  options.num_bins,
                  ) )

        c = CountingResults( labels )
        c.mObservedCounts = counter( labels, options.num_bins, options.resolution, dtype = dtype )
        
        simulated_counts = []
        for x in range(options.num_samples):
            simulated_counts.append( counter( labels, options.num_bins, options.resolution, dtype = dtype ) )
        c.mSimulatedCounts = simulated_counts
        c.mName = c.mObservedCounts.mName

        counters.append( c )

        E.info( "allocated memory successfully" )
        
    segments_per_workspace = []
    segment_sizes = []
    segments_per_label = collections.defaultdict( int )
    workspaces_per_label = collections.defaultdict( int )

    ############################################
    # get observed and simpulated counts
    nworkspaces, nempty_workspaces, nempty_contigs, nmiddle = 0, 0, 0, 0
    iteration2 = 0
    for contig, vv in workspace.iteritems():
        
        iteration2 += 1
        E.info( "counting %i/%i: %s %i segments" % \
                    ( iteration2, 
                      len(workspace), 
                      contig, 
                      len(vv) ) )

        if len(vv) == 0: continue

        iteration1 = 0
        for work_start, work_end, v in vv:
            
            left_labels, right_labels = v[0], v[1]

            iteration1 += 1

            # ignore empty segments
            if contig not in indexed_segments: 
                nempty_contigs += 1
                continue 

            r = indexed_segments[contig].find( work_start, work_end )
            segments_per_workspace.append( len(r) )

            if not r:
                nempty_workspaces += 1
                continue
            
            # collect segments and stats
            nworkspaces += 1
            observed = [(x.start, x.end) for x in r ]
            observed.sort()
            segments_per_workspace.append( len(observed) )
            segment_sizes.extend( [x[1] - x[0] for x in observed] )

            # collect basic counts
            for label in list(left_labels) + list(right_labels):
                workspaces_per_label[label] += 1
                segments_per_label[label] += len(observed)

            # add observed counts
            for counter in counters:
                counter.mObservedCounts.addCounts( observed, work_start, work_end, left_labels, right_labels )

            # create sampler
            s = sampler( observed, work_start, work_end )

            # add simulated counts
            for iteration in range(options.num_samples):
                simulated = s.sample()
                for counter in counters:
                    counter.mSimulatedCounts[iteration].addCounts( simulated, work_start, work_end, left_labels, right_labels )

    E.info( "counting finished" )
    E.info( "nworkspaces=%i, nmiddle=%i, nempty_workspaces=%i, nempty_contigs=%i" % (nworkspaces, nmiddle, nempty_workspaces, nempty_contigs) )

    ######################################################
    ## transform counts

    if options.transform_counts == "cumulative":
        transform = cumulative_transform
    elif options.transform_counts == "raw":
        transform = normalize_transform

    ####################################################
    ## analysis

    if "proximity" in options.analysis:
        outfile_proximity = E.openOutputFile( "proximity" )
        outfile_proximity.write( "\t".join( ( "label","observed","pvalue",
                                              "expected","CIlower","CIupper","qvalue","segments","workspaces" )) + "\n" )
    else:
        outfile_proximity = None

    if "area-under-curve" in options.analysis:
        outfile_auc = E.openOutputFile( "auc" )
        outfile_auc.write( "label\tobserved\texpected\tCIlower\tCIupper\n" )
    else:
        outfile_auc = None

    ## qvalue: expected false positives at p-value
    ## qvalue = expected false positives / 
    if options.do_fdr:
        E.info( "computing pvalues for fdr" )
        for counter in counters:
            for label in labels:
                E.info( "working on counter:%s label:%s" % (counter, label))

                # collect all P-Values of simulated results to compute FDR
                sim_pvalues = []
                medians = counter.getMedians( label )
                
                for median in medians:
                    pvalue = float(scipy.stats.percentileofscore( medians, median )) / 100.0 
                    sim_pvalues.append( pvalue )
                    
        sim_pvalues.sort()
    else:
        sim_pvalues = []

    ## compute observed p-values
    for counter in counters:
        counter.update()
        
    obs_pvalues = []
    for counter in counters:
        for label in labels:
            obs_pvalues.append( counter.mStats[label].pvalue )
        obs_pvalues.sort()

    ## compute observed p-values
    if options.do_fdr:
        for counter in counters:
            counter.updateFDR( obs_pvalues, sim_pvalues )

    for counter in counters:

        outofbounds_sim, totals_sim = 0, 0
        outofbounds_obs, totals_obs = 0, 0
        for label in labels:
            for sample in range(options.num_samples):
                if counter.mSimulatedCounts[sample].mOutOfBounds[label]:
                    E.debug("out of bounds: sample %i, label %s, counts=%i" % \
                                ( sample, label, counter.mSimulatedCounts[sample].mOutOfBounds[label]) )
                    outofbounds_sim += counter.mSimulatedCounts[sample].mOutOfBounds[label]
                totals_sim += counter.mSimulatedCounts[sample].mTotals[label]

            outofbounds_obs += counter.mObservedCounts.mOutOfBounds[label]
            totals_obs += counter.mObservedCounts.mTotals[label]

        E.info( "out of bounds observations: observed=%i/%i (%5.2f%%), simulations=%i/%i (%5.2f%%)" % \
                (outofbounds_obs, totals_obs,
                 100.0 * outofbounds_obs / totals_obs,
                 outofbounds_sim, totals_sim,
                 100.0 * outofbounds_sim / totals_sim,
                 ) )

        for label in labels:
            
            if outfile_auc:
                mmin, mmax, mmean = counter.getEnvelope( label, transform = normalize_transform )
                obs = normalize_transform(counter.mObservedCounts[label], counter.mObservedCounts.mOutOfBounds[label])

                def block_iterator( a1, a2, a3, num_bins ):
                    x = 0
                    while x < num_bins:
                        while x < num_bins and a1[x] <= a2[x]: x += 1
                        start = x
                        while x < options.num_bins and a1[x] > a2[x]: x += 1
                        end = x
                        total_a1 = a1[start:end].sum()
                        total_a3 = a3[start:end].sum()
                        if total_a1 > total_a3:
                            yield (total_a1 - total_a3, start,end,total_a1, total_a3)

                blocks =  list(block_iterator( obs, mmax, mmean, options.num_bins))

                if options.output_all:
                    for delta, start,end,total_obs,total_mean in blocks:
                        if end - start <= 1: continue
                        outfile_auc.write( "%s\t%i\t%i\t%i\t%f\t%f\t%f\t%f\t%f\n" %\
                                               ( label,
                                                 start * options.resolution, 
                                                 end * options.resolution, 
                                                 (end-start) * options.resolution,
                                                 total_obs, 
                                                 total_mean, 
                                                 delta,
                                                 total_obs / total_mean,
                                                 100.0 * (total_obs / total_mean - 1.0) ) )

                # output best block
                blocks.sort()
                delta, start,end,total_obs,total_mean = blocks[-1]

                outfile_auc.write( "%s\t%i\t%i\t%i\t%f\t%f\t%f\t%f\t%f\n" %\
                                        ( label,
                                          start * options.resolution, 
                                          end * options.resolution, 
                                          (end-start) * options.resolution,
                                          total_obs, 
                                          total_mean, 
                                          delta,
                                          total_obs / total_mean,
                                          100.0 * (total_obs / total_mean - 1.0) ) )

            if outfile_proximity:

                # find error bars at median
                st = counter.mStats[label]
                outfile_proximity.write( "%s\t%i\t%f\t%i\t%i\t%i\t%s\t%i\t%i\n" %\
                                             ( label,
                                               st.observed * options.resolution,
                                               st.pvalue,
                                               st.expected * options.resolution,
                                               st.ci95lower * options.resolution,
                                               st.ci95upper * options.resolution,
                                               IOTools.prettyFloat( st.qvalue ),
                                               segments_per_label[label],
                                               workspaces_per_label[label],
                                               ) )

    if options.plot:
        
        for counter in counters:
            plotCounts( counter, options, transform )

        # plot summary stats
        plt.figure()
        plt.title("distribution of workspace length")
        data = []
        for contig, segs in workspace.iteritems():
            if len(segs) == 0: continue
            data.extend( [ x[1] - x[0] for x in segs ] )

        vals, bins = numpy.histogram( data, bins=numpy.arange(0,max(data),100), new = True)

        t = float(sum(vals))
        plt.plot( bins[:-1], numpy.cumsum( vals ) / t )
        plt.gca().set_xscale('log')
        plt.legend()
        t = float(sum(vals))
        plt.xlabel( "size of workspace")
        plt.ylabel( "cumulative relative frequency")
        if options.hardcopy:
            plt.savefig( os.path.expanduser(options.hardcopy % "workspace_size") )


        plt.figure()
        plt.title("segments per block" )
        vals, bins = numpy.histogram( segments_per_workspace, bins=numpy.arange(0,max(segments_per_workspace),1), new = True)
        plt.plot( bins[:-1], vals )
        plt.xlabel( "segments per block")
        plt.ylabel( "absolute frequency")
        if options.hardcopy:
            plt.savefig( os.path.expanduser(options.hardcopy % "segments_per_block") )

        plt.figure()
        plt.title("workspaces per label" )
        plt.barh( range(0,len(labels) ), [ workspaces_per_label[x] for x in labels ], height=0.5 )
        plt.yticks( range(0,len(labels) ), labels )
        plt.ylabel( "workspaces per label")
        plt.xlabel( "absolute frequency")
        plt.gca().set_xscale('log')

        if options.hardcopy:
            plt.savefig( os.path.expanduser(options.hardcopy % "workspaces_per_label") )

        plt.figure()
        plt.title("segments per label" )
        plt.barh( range(0,len(labels) ), [ segments_per_label[x] for x in labels ], height=0.5 )
        plt.yticks( range(0,len(labels) ), labels )
        plt.ylabel( "segments per label")
        plt.xlabel( "absolute frequency")
        plt.xticks( range(0,len(labels) ), labels )
        if options.hardcopy:
            plt.savefig( os.path.expanduser(options.hardcopy % "segments_per_label") )

        if not options.hardcopy:
            plt.show()

    E.Stop()

if __name__ == "__main__":
    sys.exit(main())




"""
GTF.py - Classes and methods for dealing with gtf formatted files
=================================================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python

The default GTF version is 2.2.
"""

import string, sys, re, copy, types, collections
import Intervals
import Genomics
import IndexedGenome
import pysam

class Error(Exception):
    """Base class for exceptions in this module."""
    def __str__(self):
        return str(self.message)
    def _get_message(self, message): return self._message
    def _set_message(self, message): self._message = message
    message = property(_get_message, _set_message)

class ParsingError(Error):
    """Exception raised for errors in the input.

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message

def toDot( v ):
    '''convert value to '.' if None'''
    if v == None: return "." 
    else: return str(v)

def quote( v ):
    '''return a quoted attribute.'''
    if type(v) in types.StringTypes:
        return '"%s"' % v
    else: 
        return str(v)

class Entry:
    """read/write gtf formatted entry.

    The coordinates are kept internally in python coordinates (0-based, open-closed), but are
    output as inclusive 1-based coordinates according to

    http://www.sanger.ac.uk/Software/formats/GFF/
    """

    def __init__(self):
        self.contig = "."
        self.source = "."
        self.feature = "."
        self.frame = "."
        self.start = 0
        self.end = 0
        self.score = "."
        self.strand = "."
        self.gene_id = None
        self.transcript_id =  None
        self.attributes = {}

    def read( self, line ):
        """read gff entry from line.
        
        <seqname> <source> <feature> <start> <end> <score> <strand> <frame> [attributes] [comments]
        """
        
        data = line[:-1].split("\t")

        try:
            (self.contig, self.source, self.feature,
             self.start, self.end, self.score, self.strand,
             self.frame ) = data[:8]
        except ValueError:
            raise ValueError( "parsing error in line `%s`" % line )

        ## note: frame might be .
        (self.start, self.end) = map(int, (self.start, self.end))
        self.start -= 1

        self.parseInfo( data[8], line )

    def parseInfo( self, attributes, line ):
        """parse attributes.
        """
        # remove comments
        attributes = attributes.split( "#" )[0]
        # separate into fields
        fields = map( lambda x: x.strip(), attributes.split(";")[:-1])
        self.attributes = {}
        
        for f in fields:
            
            d = map( lambda x: x.strip(), f.split(" "))
            
            n,v = d[0], d[1]
            if len(d) > 2: v = d[1:]

            if v[0] == '"' and v[-1] == '"':
                v = v[1:-1]
            else:
                ## try to convert to a value
                try:
                    v = float( v )
                    v = int( v )
                except ValueError:
                    pass
                except TypeError:
                    pass
                
            if n == "gene_id": 
                self.gene_id = v     
            elif n == "transcript_id": 
                self.transcript_id = v
            else: 
                self.attributes[n] = v

        if not self.gene_id:
            raise ParsingError( "missing attribute 'gene_id' in line %s" % line)
        if not self.transcript_id:
            raise ParsingError( "missing attribute 'transcript_id' in line %s" % line)

    def getAttributeField( self, full = True ):
        aa = []
        for k,v in self.attributes.items():
            if type(v) == types.StringType:
                aa.append( '%s "%s"' % (k,v) )
            else:
                aa.append( '%s %s' % (k,str(v)) )

        if full:
            return"; ".join( ['gene_id "%s"' % self.gene_id, 
                                     'transcript_id "%s"' % self.transcript_id ] +\
                                        aa )
        else:
            return "; ".join( aa )

        return attributes

    def __cmp__(self, other):
        ## note: does compare by strand as well!
        return cmp( (self.contig, self.strand, self.start),
                    (other.contig, other.strand, other.start))

    def __str__(self):
        
        def _todot( val ):
            if val == None: return "."
            else: return str(val)

        attributes = self.getAttributeField( )
        return "\t".join(map(str, (self.contig, self.source,
                                   self.feature, 
                                   self.start + 1, self.end,
                                   _todot(self.score), 
                                   self.strand, 
                                   self.frame,
                                   attributes))) + ";" 

    def invert( self, lcontig ):
        """invert genomic coordinates from
        forward to reverse coordinates and back.
        """

        if self.strand in ("-", "0", "-1"):
            x = min(self.start, self.end)
            y = max(self.start, self.end)
                
            self.start = lcontig - y
            self.end   = lcontig - x

    def fromGTF( self, other, gene_id, transcript_id ):
        """fill from other entry."""
        self.contig = other.contig
        self.source = other.source
        self.feature = other.feature
        self.start = other.start
        self.end = other.end
        self.score = other.score
        self.strand = other.strand
        self.frame = other.frame
        self.gene_id = gene_id
        self.transcript_id = transcript_id
        return self

    def copy( self, other ):
        """fill from other entry.
        works both if other is :class:`GTF.Entry` or 
        :class:`pysam.GTFProxy`
        """
        self.contig = other.contig
        self.source = other.source
        self.feature = other.feature
        self.start = other.start
        self.end = other.end
        self.score = other.score
        self.strand = other.strand
        self.frame = other.frame
        self.gene_id = other.gene_id
        self.transcript_id = other.transcript_id
        self.attributes = copy.copy(other.asDict())
        # from gff - remove gene_id and transcript_id from attributes
        try:
            del self.attributes["gene_id"]
            del self.attributes["transcript_id"]
        except KeyError:
            pass

        return self

    def asDict( self ):
        '''return attributes as a dictionary.'''
        return self.attributes

    def clearAttributes( self ):
        self.attributes = {}

    def addAttribute( self, key, value = None):
        self.attributes[key] = value

    def __cmp__(self, other):
        ## note: does compare by strand as well!
        return cmp( (self.contig, self.strand, self.start),
                    (other.contig, other.strand, other.start))

    def __setitem__(self, key, value):
        self.addAttribute( key, value )
        
    def __getitem__(self, key):
        return self.attributes[key]

    def __contains__(self, key):
        return key in self.attributes
        
    def hasOverlap( self, other, min_overlap = 0 ):
        """returns true, if overlap with other entry.
        """
        return (self.contig == other.contig and self.strand == other.strand and \
                    min(self.end, other.end) - max(self.start, other.start) > min_overlap)

    def isIdentical( self, other, max_slippage = 0 ):
        """returns true, if self and other overlap completely.
        """
        return (self.contig == other.contig and \
                    self.strand == other.strand and \
                    abs(self.end - other.end) < max_slippage and \
                    abs(self.start - other.start < max_slippage) )

    def isHalfIdentical( self, other, max_slippage = 0 ):
        """returns true, if self and other overlap.
        """
        return (self.contig == other.contig and \
                    self.strand == other.strand and \
                    ( abs(self.end - other.end) < max_slippage or \
                          abs(self.start - other.start < max_slippage) ) )

def Overlap( entry1, entry2, min_overlap = 0 ):
    """returns true, if entry1 and entry2 overlap.
    """

    return (entry1.contig == entry2.contig and entry1.strand == entry2.strand and \
            min(entry1.end, entry2.end) - max(entry1.start, entry2.start) > min_overlap)

def Identity( entry1, entry2, max_slippage = 0 ):
    """returns true, if entry1 and entry2 overlap.
    """

    return (entry1.contig == entry2.contig and \
            entry1.strand == entry2.strand and \
            abs(entry1.end - entry2.end) < max_slippage and \
            abs(entry1.start - entry2.start < max_slippage) )

def HalfIdentity( entry1, entry2, max_slippage = 0 ):
    """returns true, if entry1 and entry2 overlap.
    """

    return (entry1.contig == entry2.contig and \
            entry1.strand == entry2.strand and \
            ( abs(entry1.end - entry2.end) < max_slippage or \
              abs(entry1.start - entry2.start < max_slippage) ) )

def asRanges( gffs, feature = None ):
    """return ranges within a set of gffs.

    Overlapping intervals are merged. 

    The returned intervals are sorted.
    """

    if type(feature) == types.StringType:
        gg = filter( lambda x: x.feature == feature, gffs )
    elif feature:
        gg = filter( lambda x: x.feature in feature, gffs )
    else:
        gg = gffs[:]
    
    r = [ (g.start, g.end) for g in gg ]
    return Intervals.combine( r )

def readFromFile( infile ):
    """read gtf from file."""
    result = []
    for gff in pysam.tabix_iterator( infile, pysam.asGTF() ):
        result.append( gff )
    return result

def CombineOverlaps( old_gff, method = "combine" ):
    """combine overlapping entries for a list of gffs.

    method can be any of combine|longest|shortest
    only the first letter is important.
    """

    old_gff.sort( lambda x,y: cmp( (x.contig, x.strand, x.start, x.end),
                                   (y.contig, y.strand, y.start, y.end) ) )
    
    new_gff = []

    last_e = old_gff[0]

    for e in old_gff[1:]:
        if not Overlap( last_e, e):
            new_gff.append( last_e )
            last_e = e
        else:
            if method[0] == "c":
                last_e.start = min( last_e.start, e.start )
                last_e.end = max( last_e.end, e.end )
                last_e.mInfo += " ; " + e.mInfo

    new_gff.append( last_e )
    
    return new_gff
    
def SortPerContig( gff ):
    """sort gff entries per contig and return a dictionary mapping a
    contig to the begin of the list."""
    map_contig2start = {}

    if len(gff) == 0: return map_contig2start
    
    gff.sort( lambda x,y: cmp( x.contig, y.contig) )

    last_contig = None
    start = 0
    for x in range( len(gff)):
        if last_contig != gff[x].contig:
            map_contig2start[last_contig] = (start, x)
            start = x
            last_contig = gff[x].contig

    map_contig2start[last_contig] = (start, x)

    return map_contig2start

####################################################
####################################################
####################################################
## Iterators
####################################################
def iterator( infile ):
    """return a simple iterator over all entries in a file."""
    return pysam.tabix_iterator(infile, pysam.asGTF())

def track_iterator( infile ):
    """a simple iterator over all entries in a file."""
    # note: taken from GFF.py
    ntracks = 0
    while 1:
        line = infile.readline()
        if not line: return
        if line[0] == "#": continue
        if len(line.strip()) == 0: continue
        if line.startswith("track"):
            ntracks += 1
            if ntracks > 1: raise ValueError( "more than one track in %s" % infile)
            continue
        gff = Entry()
        gff.read( line )
        yield gff

def chunk_iterator( gff_iterator ):
    """iterate over the contents of a gff file.

    return entries as single element lists
    """
    for gff in gff_iterator:
        yield [gff,]

def iterator_contigs( gffs ):
    """iterate over contigs.
    
    TODO: implement as coroutines
    """

    last_contig, data = None, []
    found = set()
    for gff in gffs:
        if last_contig != gff.contig:
            if last_contig: yield last_contig, data
            last_contig = gff.contig
            assert last_contig not in found, "input not sorted by contig."
            found.add(last_contig) 

        data.append( gff )

    if last_contig: yield last_contig, data

def transcript_iterator(gff_iterator, strict = True ):
    """iterate over the contents of a gtf file.

    return a list of entries with the same transcript id.

    Note: the entries for the same transcript have to be consecutive 
    in the file.
    """
    last = None
    matches = []
    found = set()

    for gff in gff_iterator:
        this = gff.transcript_id + gff.gene_id
        if last != this:
            if last: yield matches
            matches = []
            assert not strict or this not in found, "duplicate entry: %s" % this
            found.add( this )
            last = this
        matches.append( gff )
    if last: yield matches

def joined_iterator(gffs, group_field = None):
    """iterate over the contents of a gff file.

    return a list of entries with the same group id.
    Note: the entries have to be consecutive in
    the file.
    """
    last_group_id = None
    matches = []

    for gff in gffs:
        
        if group_field:
            group_id = gff.fields[group_field]
        else:
            group_id = gff.attributes
                
        if last_group_id != group_id:
            if last_group_id: yield matches
            matches = []
            last_group_id = group_id
                
        matches.append( gff )

    if last_group_id: yield matches

def gene_iterator( gff_iterator, strict = True ):
    """iterate over the contents of a gtf file.

    return a list of transcripts with the same gene id. 

    Note: the entries have to be consecutive in the file, i.e,
    first sorted by transcript and then by gene id.

    Genes with the same name on different contigs are resolved
    separately in *strict* = False.
    """
    last = Entry()
    matches = []
    found = set()
    for gffs in transcript_iterator( gff_iterator, strict ):
        
        if last.gene_id != gffs[0].gene_id or last.contig != gffs[0].contig:
            if matches: yield matches
            matches = []
            last = gffs[0]
            assert not strict or last not in found, "duplicate entry %s" % last 
            found.add(last.gene_id)

        matches.append( gffs )

    if matches: yield matches

def flat_gene_iterator( gff_iterator, strict = True ):
    """iterate over the contents of a gtf file.

    return a list of entries with the same gene id.

    Note: the entries have to be consecutive in the file, i.e,
    sorted by gene_id

    Genes with the same name on different contigs are resolved
    separately in *strict* = False

    """

    last = Entry()
    matches = []
    found = set()
    for gff in gff_iterator:
        
        if last.gene_id != gff.gene_id or last.contig != gff.contig:
            if matches: yield matches
            matches = []
            last = gff
            if strict:
                assert last not in found, "duplicate entry %s" % last.gene_id
                found.add(last.gene_id)
        matches.append( gff )

    if matches: yield matches

def merged_gene_iterator( gff_iterator ):
    """iterate over the contents of a gtf file.

    Each gene is merged into a single entry spanning the whole
    stretch that a gene covers.

    Note: the entries have to be consecutive in the file, i.e,
    sorted by gene_id
    """
    for m in flat_gene_iterator( gff_iterator ):
        gff = Entry()
        gff.copy( m[0] )
        gff.start = min( [x.start for x in m ] )
        gff.end = max( [x.end for x in m ] )
        yield gff

def iterator_filtered( gff_iterator, feature = None, source = None, contig = None, interval = None, strand = None):
    """iterate over the contents of a gff file.

    yield only entries for a given feature
    """
    if interval: start, end = interval
    if strand == ".": strand = None

    for gff in gff_iterator:
        if feature and gff.feature != feature: continue
        if source and gff.source != source: continue
        if contig and gff.contig != contig: continue
        if strand and gff.strand != strand: continue
        if interval and min(end,gff.end) - max(start,gff.start) < 0: continue
        yield gff

def iterator_sorted_chunks( gff_iterator, sort_by = "contig-start" ):
    """iterate over chunks in a sorted order

    sort_by can be 
    
    contig-start
       sort by position ignoring the strand
    contig-strand-start
       sort by position taking the strand into account
    
    returns the chunks. 
    """
    
    ## get all chunks and annotate with sort order
    if sort_by == "contig-start":
        chunks = ([ (x[0].contig, min( [y.start for y in x] ), x) for x in gff_iterator ] )
        chunks.sort()
        for contig, start, chunk in chunks:
            chunk.sort( key = lambda x: (x.contig, x.start) )
            yield chunk
    elif sort_by == "contig-strand-start":
        chunks = ([ (x[0].contig, x[0].strand, min( [y.start for y in x] ), x) for x in gff_iterator ] )
        chunks.sort()
        for contig, start, strand, chunk in chunks:
            chunk.sort( key = lambda x: (x.contig, x.strand, x.start) )
            yield chunk
    else:
        raise ValueError( "unknown sort order %s" % sort_by )

def iterator_min_feature_length( gff_iterator, min_length, feature="exon" ):
    """select only those genes with a minimum length of a given feature."""
    for gffs in gff_iterator:
        intervals = [ (x.start, x.end) for x in gffs if x.feature == feature ]
        intervals = Intervals.combine( intervals )
        t = sum( ( x[1] - x[0] for x in intervals ) )
        if t >= min_length: yield gffs

def iterator_sorted( gff_iterator, sort_order = "gene" ):
    '''sort input and yield sorted output.'''
    entries = list(gff_iterator)
    if sort_order == "gene":
        entries.sort( key = lambda x: (x.gene_id, x.transcript_id, x.contig, x.start) )
    elif sort_order == "gene":
        entries.sort( key = lambda x: (x.gene_id, x.contig, x.start) )
    elif sort_order == "contig+gene":
        entries.sort( key = lambda x: (x.contig,x.gene_id,x.transcript_id,x.start) )
    elif sort_order == "transcript":
        entries.sort( key = lambda x: (x.transcript_id, x.contig, x.start) )
    elif sort_order == "position":
        entries.sort( key = lambda x: (x.contig, x.start) )
    elif sort_order == "position+gene":
        entries.sort( key = lambda x: (x.gene_id, x.start) )
        genes = list( flat_gene_iterator(entries) )
        genes.sort( key = lambda x: (x[0].contig, x[0].start) )
        entries = IOTools.flatten( genes )

    for entry in entries:
        yield entry

def toIntronIntervals( chunk ):
    '''convert a set of gtf elements within a transcript to intron coordinates.

    Will raise an error if more than one transcript is submitted.
    
    Note that coordinates will still be forward strand coordinates
    '''
    if len(chunk) == 0: return []
    t = set([ x.transcript_id for x in chunk ])
    contig, strand, transcript_id = chunk[0].contig, chunk[0].strand, chunk[0].transcript_id
    for gff in chunk:
        assert gff.strand == strand, "features on different strands."
        assert gff.contig == contig, "features on different contigs."
        assert gff.transcript_id == transcript_id, "more than one transcript submitted"

    intervals = Intervals.combine( [ (x.start, x.end) for x in chunk ] )
    return Intervals.complement( intervals )
    
def toSequence( chunk, fasta ):
    """convert a list of gff attributes to a single sequence.
    
    This function ensures correct in-order concatenation on
    positive/negative strand. Overlapping regions are merged.
    """
    if len(chunk) == 0: return ""

    contig, strand = chunk[0].contig, chunk[0].strand 

    for gff in chunk:
        assert gff.strand == strand, "features on different strands."
        assert gff.contig == contig, "features on different contigs."
    
    intervals = Intervals.combine( [ (x.start, x.end) for x in chunk ] )
    lcontig = fasta.getLength(contig)
    positive = Genomics.IsPositiveStrand( strand )

    if not positive: 
        intervals = [ (lcontig - end, lcontig - start) for start,end in intervals ]
        intervals.reverse()

    s = [ fasta.getSequence( contig, strand, start, end ) for start, end in intervals ]

    return "".join(s)

##----------------------------------------------------------------------------------------
def iterator_overlapping_genes( gtf_iterator, min_overlap = 0 ):
    """return overlapping genes."""
    for gene in flat_gene_iterator(gtf_iterator):
        gene.sort( key=lambda x: x.start )
        genes.append( gene[0].contig, gene[0].start, gene )
        
    genes.sort()

    contig, last_end, ovl = None, 0, 0, []

    for this_contig, this_start, gene in genes:
        if this_contig != last_contig or last_end < this_start:
            if last_contig: yield ovl
            ovl, last_contig = [], this_contig
            last_end = 0

        last_end = max( last_end, gene[-1].end )
       
    if last_contig: yield ovl



##----------------------------------------------------------------------------------------
def iterator_transcripts2genes( gtf_iterator, min_overlap = 0 ):
    """cluster transcripts by exon overlap.

    The gene id is set to the first transcript encountered of a gene.
    If a gene stretches over several contigs, subsequent copies are
    appended a number.
    """

    map_transcript2gene = {}
    gene_ids = collections.defaultdict( list )

    for chunk in iterator_overlaps( gtf_iterator ):
        transcript_ids = list(set( [x.transcript_id for x in chunk ] ))
        contig = chunk[0].contig

        # have any of these already encountered?
        for x in transcript_ids:
            if x in map_transcript2gene:
                gene_id = map_transcript2gene[x]
                break
        else:
            # arbitrarily pick one
            gene_id = transcript_ids[0]

        if gene_id not in gene_ids:
            gene_ids[gene_id].append( contig )
            index = 0
        else:
            try:
                index = gene_ids[gene_id].index(contig)
            except ValueError:
                index = len(gene_ids[gene_id])
                gene_ids[gene_id].append( contig )

        for x in transcript_ids: map_transcript2gene[x] = gene_id
        
        if index: gene_id += ".%i" % index
        for x in chunk: x.gene_id = gene_id
        yield chunk

def readAsIntervals( gff_iterator, 
                     with_values = False, 
                     with_records = False,
                     merge_genes = False,
                     with_gene_id = False,
                     with_transcript_id = False,
                     use_strand = False ):
    """read tuples of (start, end) from a GTF file.

    If with_values is True, a value is added to the tuples.
    If with_records is True, the record is added to the tuples.
    If with_gene_id is True, the gene_id is added to the tuples
    with_values and with_records are exclusive.
    
    Ignores strand and everything but the coordinates.

    If use_strand is True, intervals will be grouped by contig and strand.
    The default is to group by contig only.

    Returns a dictionary of intervals by contig.
    """

    assert not (with_values and with_records), "both with_values and with_records are true."
    intervals = collections.defaultdict( list )

    if merge_genes:
        it = merged_gene_iterator( gff_iterator )
    else:
        it = gff_iterator
        
    if use_strand:
        keyf = lambda x: (x.contig, x.strand)
    else:
        keyf = lambda x: x.contig

    if with_values:
        for gff in it:
            intervals[keyf(gff)].append( (gff.start,gff.end,gff.score) )
    elif with_records:
        for gff in it:
            intervals[keyf(gff)].append( (gff.start,gff.end,gff) )
    elif with_gene_id:
        for gff in it:
            intervals[keyf(gff)].append( (gff.start,gff.end,gff.gene_id) )
    elif with_transcript_id:
        for gff in it:
            intervals[keyf(gff)].append( (gff.start,gff.end,gff.transcript_id) )
    else:
        for gff in gff_iterator:
            intervals[keyf(gff)].append( (gff.start,gff.end) )
        
    return intervals

def iterator_overlaps( gff_iterator, min_overlap = 0 ):
    """iterate over gff file and return a list of features that
    are overlapping.

    The input should be sorted by contig,start
    """
    
    last = gff_iterator.next()
    matches = [last]
    end = last.end
    for this in gff_iterator:
        if last.contig != this.contig or \
                end - this.start <= min_overlap:
            yield matches
            matches = [this]
            end = this.end
            last = this
            continue

        assert last.start <= this.start, "input file needs to be sorted by contig, start:\n%s\n%s\n" % (str(last), str(this) )
        matches.append( this )
        last = this
        end = max( end, this.end )

    yield matches

def readAndIndex( iterator, with_value = True ):
    '''read from gtf stream and index.

    returns an :class:`IndexedGenome.IndexedGenome`
    '''

    if with_value:
        index = IndexedGenome.IndexedGenome()
        for gtf in iterator:
            index.add( gtf.contig, gtf.start, gtf.end, gtf )
    else:
        index = IndexedGenome.Simple()
        for gtf in iterator:
            index.add( gtf.contig, gtf.start, gtf.end )

    return index

def getGene2Transcript( iterator ):
    '''return a dictionary mapping a gene to its transcripts.'''
    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: bams2correlation.py 2861 2010-02-23 17:36:32Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
bams2correlation.py
===================

:Author: Andreas Heger
:Release: $Id: bams2correlation.py 2861 2010-02-23 17:36:32Z andreas $
:Date: |today|
:Tags: Python

Purpose
-------

Compare per base coverage between BAM files.

Usage
-----

Example::

   python bams2correlation.py in1.bam in2.bam -g reference.fasta

This command generates a tab delimited output with columns chromosome, base coordinate, number of overlapping reads
in in1.bam, and number of overlapping reads in in2.bam.

Type::

   python <script_name>.py --help

for command line help.

Documentation
-------------

This tools allows users to obtain per base coverage over the genome for one or more BAM files.  The output includes
all bases in the supplied reference fasta except those with no coverage in the input BAM or BAMs.

At present the --interval or -i option has not been implemented.  

Code
----

""" 

import os
import sys
import re
import optparse
import collections

import CGAT.Experiment as E
import pysam
import CGAT.IndexedFasta as IndexedFasta

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: bams2correlation.py 2861 2010-02-23 17:36:32Z andreas $", usage = globals()["__doc__"] )

    parser.add_option("-i", "--intervals", dest="filename_intervals", type="string",
                      help="filename with intervals to use [default=%default]."  )
    parser.add_option("-g", "--genome-file", dest="genome_file", type="string",
                      help="filename with genome [default=%default]."  )

    parser.set_defaults(
        filename_intervals = None,
        genome_file = None,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    if len(args) < 1: raise ValueError("please supply at least two BAM files.")

    samfiles = []
    for f in args:
        samfiles.append( pysam.Samfile( f, "rb" ) )
        
    if options.filename_intervals:
        raise NotImplementedError( "It is not yet possible to specify intervals of interest.  Repeat command without intervals option." )
    
    if options.genome_file:
        fasta = IndexedFasta.IndexedFasta( options.genome_file )

    options.stdout.write( "contig\tpos\t%s\n" % "\t".join(args))

    ninput, nskipped, noutput = 0, 0, 0
    for contig in fasta.getContigs():
        
        missing_contig = False

        positions = {}

        # lazy way: use dictionary
        for x, f in enumerate(samfiles): 
            try:
                i = f.pileup( contig )
            except ValueError:
                missing_contig = True
                break

            for v in i: 
                vp = v.pos
                if vp in positions:
                    positions[vp].append( v.n )
                else:
                    positions[vp] = [0] * x + [v.n]

            # fill with 0 those not touched in this file
            for p in positions.keys():
                if len(positions[p]) <= x:
                    positions[p].append( 0 )

        if missing_contig:
            nskipped += 1
            continue

        noutput += 1
        for pos in sorted(positions.keys()):
            vals = positions[pos]
            options.stdout.write( "%s\t%i\t%s\n" % (contig, pos, 
                                                    "\t".join(map(str, vals))))

    E.info( "ninput=%i, noutput=%i, nskipped=%i" % (ninput, noutput,nskipped) )

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
SuffixArray.py - sarry frontend
===============================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''

import os, string, sys

ParamExecutableLookup   = "sary"

def Search( pattern, suffix_arrays ):

    matches = []

    for suffix_array in suffix_arrays:
        statement = "%s %s %s" % (ParamExecutableLookup, pattern, suffix_array)
        matches += map( lambda x: x[:-1], os.popen( statement ).readlines())
        
    return matches
    




################################################################################
#   Gene prediction pipeline 
#
#   $Id: plot_tree.py 2782 2009-09-10 11:40:29Z andreas $
#
#   Copyright (C) 2004 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
tree2svg.py - plot a tree in svg format
========================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

plot a tree in New-Hampshire format. The output is svg.

Usage
-----

Type::

   python <script_name>.py --help

for command line help.

Examples
--------

The following plots a tree with branch length from :file:`ks.tree` adding
branch length information from :file:`kaks.tree`, `ks2.tree` and `kaks2.tree`::

   cat ks.tree kaks.tree ks2.tree kaks2.tree |\
   python plot_tree.py --verbose=0 --show-branchlengths --annotation=kaks --annotation=master --annotation=kaks --font-style-tips=italic > tree.svg

Adding tables to trees
----------------------

Branches can be decorated with tables, for example::

   cat test.tree tables.tree | python plot_tree.py -v 0 --filename-tables=test.table --annotation=tables

Here, the file :file:`test.tree` is::

   (((A:1,B:1):1,C:2),D:3);

The file :file:`tables.tree` links branches to tables. The table number is given as the branch length::

   (((A:5,B:1):2,C:3),D:4);

Note that table ids need to start with 1, but need not be consecutive numbers. Finally, the tables are
in the file :file:`test.table` given as argument to the options ``--filename-tables``.

   table=3
   header1 header2 header3
   0row1   col1    col2
   0row2   col1    col2
   table=1
   header1 header2 header3
   1row1    col1    col2
   1row2    col1    col2
   table=2
   header1 header2 header3
   2row1    col1    col2
   2row2    col1    col2

A table is started by the line ``table=#`` where ``#`` is an integer number. The actual table follows as
a tab-separated table with the first line being interpreted as a header. Lines starting with ``#`` are
ignored.

Code
----
'''

import os
import sys
import string
import re
import getopt
import time
import optparse
import math
import tempfile
import subprocess

from types import *

import CGAT.Experiment as E
import CGAT.TreeTools as TreeTools
import CGAT.IOTools as IOTools
import CGAT.SVGTree as SVGTree
import CGAT.SVGdraw as SVGdraw


###################################################################
###################################################################
###################################################################
##
###################################################################
class NodeDecoratorSupportPieChart(SVGTree.NodeDecorator):
    """class for decorating internal nodes using a
    pie-chart.

    Uses the support information in the tree.
    """
    
    mRadius = 40
    mFontSize = 12
    mFont = "Verdana"
    mSeparator = 5
    mStrokeWidth = 1
    
    def __init__(self, tree, *args, **kwargs):
        
        SVGTree.NodeDecorator.__init__( self, tree, *args, **kwargs)

    def getElements( self, node_id, x, y):

        node = self.mTree.node(node_id)
        e = []

        p = node.data.support

        if p == 1.0:
            e.append( SVGdraw.circle( x, y, self.mRadius,
                                      stroke = "rgb(%i,%i,%i)" % SVGTree.BLACK,
                                      fill = "rgb(%i,%i,%i)" % SVGTree.RED))
        elif p > 0.0:
            e.append( SVGdraw.circle( x, y, self.mRadius,
                                      stroke = "rgb(%i,%i,%i)" % SVGTree.BLACK,
                                      fill = "rgb(%i,%i,%i)" % SVGTree.WHITE))

            d = SVGdraw.pathdata( x, y )

            d.line( x + self.mRadius, y )

            angle = 360.0 * p
            dx = self.mRadius * math.cos( math.radians(angle) ) + x
            dy = self.mRadius * math.sin( math.radians(angle) ) + y

            if p <= 0.5:
                d.ellarc( self.mRadius, self.mRadius, 0, 0, 1, dx, dy )
            else:
                d.ellarc( self.mRadius, self.mRadius, 0, 1, 1, dx, dy )            

            e.append( SVGdraw.path( d,
                                    stroke = "rgb(%i,%i,%i)" % SVGTree.RED,
                                    fill = "rgb(%i,%i,%i)" % SVGTree.RED,
                                    stroke_width = self.mStrokeWidth ) )

        else:
            pass

        return e

    def getHeight( self, node_id ):
        return 0

    def getWidth( self, node_id ):
        return 0

###################################################################
###################################################################
###################################################################
##
###################################################################
class BranchDecoratorTable( SVGTree.BranchDecoratorHorizontal ):
    """branch decorator - add a table onto a branch length.
    
    The table will be plotted below each branch.

    This decorator requires labeled branches within a tree.
    """

    mBranchLengthFormat = "%5.2f"
    mFontSize = 10
    
    def __init__(self, tree, filename, *args, **kwargs):
        SVGTree.BranchDecoratorHorizontal.__init__( self, tree, *args, **kwargs)
        self.mWritten = 0

        infile = open(filename, "r")
        self.mTables = {}
        table_id = None        
        for line in infile:
            if line.startswith("#"): continue
            if line.startswith("table="):
                if table_id: self.mTables[table_id] = table
                table_id = re.search("table=(\d+)", line).groups()[0]
                if int(table_id) == 0: raise ValueError( "table id 0 is invalid" )
                table = []
                continue
            table.append( line[:-1].split("\t") )
        if table_id: self.mTables[table_id] = table

        self.mColumnWidths = {}
        
        for id, table in self.mTables.iteritems():
            if len(table) == 0:
                raise ValueError("empty table %s" % id)
            
            column_widths = [0] * len( table[0] )
            
            for row in table:
                if len(column_widths) != len(row):
                    raise ValueError("table %s has unequal row lengths" % id )

                for x, col in enumerate(row):
                    column_widths[x] = max( column_widths[x], len(col) )
            
            self.mColumnWidths[id] = column_widths

    def getElements( self, node_id, x1, x2, y ):

        e = SVGTree.BranchDecoratorHorizontal.getElements( self, node_id, x1, x2, y )

        table_id = str(int(self.mTree.node(node_id).data.branchlength))

        if table_id not in self.mTables:
            return e

        startx = x1 + self.mFontSize
        y = y + self.mFontSize

        table, column_widths = self.mTables[table_id], self.mColumnWidths[table_id]

        font_weight = "bold"
        
        for r, row in enumerate(table):
            x = startx
            
            for c, col in enumerate(row):
                e.append( SVGdraw.text( x, y,
                                        col,
                                        20, 
                                        self.mFont,
                                        stroke = "rgb(%i,%i,%i)" % self.mFontColour,
                                        font_weight = font_weight,
                                        text_anchor = "left" ))
                x += column_widths[c] * self.mFontSize // 2
                
            y +=  self.mFontSize
            font_weight = "normal"
            
        return e
    
    def getHeight( self, node_id ):
        
        table_id = str(int(self.mTree.node(node_id).data.branchlength))

        if table_id in self.mTables:
            table_height = len( self.mTables[table_id]) * self.mFontSize
        else:
            table_height = 0
        
        return 5 + table_height
        
    def getWidth( self, node_id ):
        table_id = str(int(self.mTree.node(node_id).data.branchlength))
        if table_id in self.mTables:
            return 5 + sum(self.mColumnWidths[table_id])

        return 5
        
###------------------------------------------------------------------------------------------------
def main():

    parser = E.OptionParser( version = "%prog version: $Id: plot_tree.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("-i", "--title", dest="title", type="string",
                      help="page title.")
    parser.add_option("-f", "--footer", dest="footer", type="string",
                      help="page footer.")
    parser.add_option("-s", "--filename-tree", dest="filename_tree", type="string",
                      help="filename with tree."  )
    parser.add_option("-t", "--tree", dest="tree", type="string",
                      help="tree."  )
    parser.add_option( "-r", "--species-regex", dest="species_regex", type="string" ,
                       help="regular expression to extract species from identifier.")
    parser.add_option( "--colour-by-species", dest="colour_by_species", action="store_true",
                      help="colour by species."  )
    parser.add_option( "--support-style", dest="support_style", type="choice",
                       choices=("pie", "number"),
                      help="style for support information."  )
    parser.add_option( "--error-style", dest="error_style", type="choice",
                       choices=("pie", "number"),
                      help="style for error information."  )
    parser.add_option( "--branch-scale", dest="branch_scale", type="float",
                      help="branch length scale factor."  )
    parser.add_option( "--height-scale", dest="height_scale", type="float",
                      help="height scale factor."  )
    parser.add_option( "-a", "--annotations", dest="annotations", type="choice", action="append",
                       choices=("support", "error", "kaks", "master", "value", "tables" ),
                       help="annotations given by further trees."  )
    parser.add_option("--filename-tables", dest="filename_tables", type="string",
                       help="add tables from file (need also set options -a tables) [%default]")
    parser.add_option("--show-branchlengths", dest="show_branchlengths", action="store_true",
                      help="show branch lengths." )
    parser.add_option("--leaf-symbol", dest="plot_leaf_symbol", type="choice",
                      choices=("square", "circle" ),
                      help="Symbol for leaves." )
    parser.add_option("--font-size-branches", dest="font_size_branches", type="int",
                      help="set font size for branches." )
    parser.add_option("--font-size-tips", dest="font_size_tips", type="int",
                      help="set font size for tips." )
    parser.add_option("--font-style-tips", dest="font_style_tips", type="choice",
                      choices=("normal", "italic",),
                      help="set font style for tips." )
    parser.add_option("--filename-map", dest="filename_map", type="string",
                      help="filename with a name translation table." )
    parser.add_option("--filename-map-species2colour", dest="filename_colour_map", type="string",
                      help="filename with a map of species to colour." )
    parser.add_option("--no-leaf-labels", dest="plot_leaf_labels", action="store_false",
                      help="do not show labels at leafs." )
    parser.add_option("--no-ruler", dest="plot_ruler", action="store_false",
                      help="do not plot ruler." )
    
    parser.set_defaults(
        titles = "",
        title = "",
        footer = "",
        filename_tree = None,
        species_regex ="^([^|]+)\|",
        colour_by_species = None,
        tree = None,
        branch_scale = 0,
        height_scale = 0,
        support_style = None,
        error_style = "number",
        kaks_style = "number",
        annotations = None,
        show_branchlengths = False,
        branch_length_format = "%5.2f",
        font_size_tips = None,
        font_size_branches = None,
        font_style_tips = None,
        filename_map = None,
        filename_colour_map = None,
        plot_leaf_labels = True,
        plot_leaf_symbol = None,
        plot_ruler = True,
        filename_tables = None,
        )

    (options, args) = E.Start( parser, add_pipe_options = True )
    
    if options.filename_tree:
        tree_lines = open(options.filename_tree, "r").readlines()
    elif options.tree:
        tree_lines = options.tree
    else:
        tree_lines = sys.stdin.readlines()

    nexus = TreeTools.Newick2Nexus( tree_lines )
    master_tree = nexus.trees[0]

    if options.filename_map:
        map_names = IOTools.ReadMap( open(options.filename_map, "r") )

        for id, node in master_tree.chain.items():
            if node.data.taxon in map_names:
                node.data.taxon = map_names[node.data.taxon]

    if options.loglevel >= 2:
        master_tree.display()
    
    plot = SVGTree.SVGTree( master_tree )

    if options.branch_scale:
        plot.setBranchScale( options.branch_scale )
        
    if options.height_scale != None:
        plot.setHeightScale( options.height_scale )    

    if options.font_size_tips != None:
        plot.setFontSize( options.font_size_tips )

    if options.plot_ruler == False:
        plot.setRulerElements( [] )

    if options.show_branchlengths:
        b = SVGTree.BranchDecoratorHorizontalBranchLength( master_tree ) 
        if options.font_size_branches:
            b.setFontSize( options.font_size_branches )
        plot.setDecoratorHorizontalBranches( b )

    if options.colour_by_species:
        if options.filename_colour_map:
            map_species2colour = IOTools.ReadMap( open( options.filename_colour_map, "r"))
        else:
            map_species2colour = None
            
        rx = re.compile(options.species_regex)
        extract_species = lambda x: rx.search( x ).groups()[0]
        plot.setDecoratorExternalNodes( SVGTree.NodeDecoratorBySpecies( master_tree,
                                                                        plot_symbol = options.plot_leaf_symbol,
                                                                        plot_label = options.plot_leaf_labels,
                                                                        map_species2colour = map_species2colour,
                                                                        extract_species= extract_species ) )

    if options.font_style_tips:
        plot.getDecoratorExternalNodes().setFontStyle( options.font_style_tips )

    plot.getDecoratorExternalNodes().setPlotLabel( options.plot_leaf_labels )

    current_tree = 1
    
    ## add annotations by further trees given on the command line
    branch_length_annotations = []

    current_reference_tree = master_tree
    
    if options.annotations:
        for annotation in options.annotations:
            
            tree = nexus.trees[current_tree]
            
            if annotation == "support":

                tree.branchlength2support()
                for id, node in tree.chain.items():
                    node.data.branchlength = 1.0

                if options.support_style == "pie":
                    plot.setDecoratorInternalNodes( NodeDecoratorSupportPieChart( nexus.trees[current_tree] ) )
                    
            elif annotation == "error":

                if options.error_style == "number":
                    b = SVGTree.BranchDecoratorHorizontalBranchLengthError( current_reference_tree, tree )
                    if options.font_size_branches:
                        b.setFontSize( options.font_size_branches )
                    branch_length_annotations.append( b )

            elif annotation == "kaks":

                if options.kaks_style == "number":
                    b = SVGTree.BranchDecoratorHorizontalBranchLengthWithKaks( current_reference_tree, tree )
                    if options.font_size_branches:
                        b.setFontSize( options.font_size_branches )
                    branch_length_annotations.append( b )                        

            elif annotation == "value":

                b = SVGTree.BranchDecoratorHorizontalBranchLength( tree )
                if options.font_size_branches:
                    b.setFontSize( options.font_size_branches )
                branch_length_annotations.append( b )                        

            elif annotation == "master":
                current_reference_tree = tree

            elif annotation == "tables":
                b = BranchDecoratorTable( tree, filename= options.filename_tables )
                plot.setDecoratorHorizontalBranches( b )
                
            current_tree += 1

        if len(branch_length_annotations) == 1:
            b = branch_length_annotations[0]
        elif len(branch_length_annotations) == 2:
            b1, b2 = branch_length_annotations
            b1.setFontColour( SVGTree.BLUE )
            b2.setFontColour( SVGTree.RED )
            b = SVGTree.BranchDecoratorHorizontalAboveBelow( master_tree, b1, b2 )
        elif len(branch_length_annotations) > 2:
            raise "obtained more than three branch length annotations. Layout not implemented"

        plot.setDecoratorHorizontalBranches( b )
    
    plot.initializePlot()
    
    plot.writeToFile(sys.stdout)
    
    E.Stop()


if __name__ == "__main__":
    sys.exit(main())
    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
index_fasta.py - Index fasta formatted files 
============================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script indexes one or more :term:`fasta` formatted files into a 
database that can be used by other scripts in the CGAT code collection
and :mod:`IndexedFasta` for quick acces to sequence fragments.

By default, the database is itself a :term:`fasta` formatted file in which
all line breaks and other white space characters have been removed.
Compression methods are available to conserve disk space, though they do 
come at a performance penalty.

See also http://pypi.python.org/pypi/pyfasta for another implementation. 
Samtools provides similar functionality with the ``samtools faidx`` command.

Examples
--------

Index a collection of fasta files::

  python index_fasta.py oa_ornAna1_softmasked ornAna1.fa.gz > oa_ornAna1_softmasked.log

To retrieve a segment::

  python index_fasta.py --extract=chr5:1000:2000 oa_ornAna1_softmasked 

Indexing from a tar file:

  python index_fasta.py oa_ornAna1_softmasked ornAna1.tar.gz > oa_ornAna1_softmasked.log

Indexing from stdin:

  zcat ornAna1.fa.gz | python index_fasta.py oa_ornAna1_softmasked - > oa_ornAna1_softmasked.log

Usage
-----

Type::

   python index_fasta.py --help

for command line help.

Command line options
--------------------

'''
import CGAT.IndexedFasta as IndexedFasta
import CGAT.Experiment as E
import sys, re, os

def main():


    parser = E.OptionParser( version = "%prog version: $Id: IndexedFasta.py 2801 2009-10-22 13:40:39Z andreas $", 
                             usage = globals()["__doc__"])

    parser.add_option( "-e", "--extract", dest="extract", type="string",
                       help="extract region for testing purposes. Format is contig:strand:from:to. "
                            "The default coordinates are 0-based open/closed coordinates on both strands. "
                            "For example, chr1:+:10:12 will return bases 11 to 12 on chr1." )

    parser.add_option( "-c", "--compression", dest="compression", type="choice",
                       choices=("lzo", "zlib", "gzip", "dictzip", "bzip2", "debug"),
                       help="compress database [default=%default]." )

    parser.add_option( "--random-access-points", dest="random_access_points", type="int",
                       help="save random access points every # number of nucleotides [default=%default]." )

    parser.add_option( "-i", "--input-format", dest="input_format", type="choice",
                       choices=("one-forward-open", "zero-both-open" ),
                       help="coordinate format of input [default=%default]." )

    parser.add_option( "-s", "--synonyms", dest="synonyms", type="string",
                       help="list of synonyms, comma separated with =, for example, chr1=chr1b [default=%default]" )

    parser.add_option( "-b", "--benchmark", dest="benchmark", action="store_true",
                       help="benchmark time for read access [default=%default]." )
    
    parser.add_option( "--benchmark-num-iterations", dest="benchmark_num_iterations", type="int",
                       help="number of iterations for benchmark [default=%default]." )

    parser.add_option( "--benchmark-fragment-size", dest="benchmark_fragment_size", type="int",
                       help="benchmark: fragment size [default=%default]." )

    parser.add_option( "--verify", dest="verify", type="string",
                       help="verify against other database [default=%default].")

    parser.add_option( "--file-format", dest="file_format", type="choice",
                       choices = ("fasta", "auto", "fasta.gz", "tar", "tar.gz"),
                       help="file format of input. Supply if data comes from stdin [default=%default].")

    parser.add_option( "-a", "--clean-sequence", dest="clean_sequence", action="store_true",
                       help="remove X/x from DNA sequences - they cause errors in exonerate [default=%default]." )

    parser.add_option( "--allow-duplicates", dest="allow_duplicates", action="store_true",
                       help="allow duplicate identifiers. Further occurances of an identifier are suffixed by an '_%i' [default=%default]." )

    parser.add_option( "--regex-identifier", dest="regex_identifier", type="string",
                       help="regular expression for extracting the identifier from fasta description line [default=%default]." )

    parser.add_option( "--compress-index", dest="compress_index", action="store_true",
                       help="compress index [default=%default]." )

    parser.add_option( "--force", dest="force", action="store_true",
                       help="force overwriting of existing files [default=%default]." )

    parser.add_option( "-t", "--translator", dest="translator", type="choice",
                       choices=("solexa", "phred", "bytes", "range200" ),
                       help="translate numerical quality scores [default=%default]." )


    parser.set_defaults(
        extract = None,
        input_format = "zero-both-open",
        benchmark_fragment_size = 1000,
        benchmark_num_iterations = 1000000,
        benchmark = False,
        compression = None,
        random_access_points = 0,
        synonyms = None,
        verify = None,
        verify_num_iterations = 100000,
        verify_fragment_size = 100,
        clean_sequence = False,
        allow_duplicates = False,
        regex_identifier = None,
        compress_index = False,
        file_format = "auto",
        force = False,
        translator = None )
    
    (options, args) = E.Start( parser )

    if options.synonyms:
        synonyms = {}
        for x in options.synonyms.split(","):
            a,b = x.split("=")
            a = a.strip()
            b = b.strip()
            if a not in synonyms: synonyms[a] = []
            synonyms[a].append( b )
    else:
        synonyms = None

    if options.translator:
        if options.translator == "phred":
            options.translator = TranslatorPhred()
        elif options.translator == "solexa":
            options.translator = TranslatorSolexa()
        elif options.translator == "bytes":
            options.translator = TranslatorBytes()
        elif options.translator == "range200":
            options.translator = TranslatorRange200()
        else:
            raise ValueError("unknown translator %s" % options.translator)
        
    if options.extract:
        fasta = IndexedFasta.IndexedFasta( args[0] )
        fasta.setTranslator( options.translator )
        converter = IndexedFasta.getConverter( options.input_format )
        
        contig, strand, start, end = IndexedFasta.parseCoordinates( options.extract )
        sequence = fasta.getSequence( contig, strand,
                                      start, end,
                                      converter = converter )
        options.stdout.write( ">%s\n%s\n" % \
                              ( options.extract, sequence ) )
    elif options.benchmark:
        import timeit
        timer = timeit.Timer( stmt="benchmarkRandomFragment( fasta = fasta, size = %i)" % (options.benchmark_fragment_size),
                              setup="""from __main__ import benchmarkRandomFragment,IndexedFasta\nfasta=IndexedFasta.IndexedFasta( "%s" )""" % (args[0] ) )

        t = timer.timeit( number = options.benchmark_num_iterations )
        options.stdout.write("iter\tsize\ttime\n" )
        options.stdout.write("%i\t%i\t%i\n" % (options.benchmark_num_iterations, options.benchmark_fragment_size, t ) )
    elif options.verify:
        fasta1 = IndexedFasta.IndexedFasta( args[0] ) 
        fasta2 = IndexedFasta.IndexedFasta( options.verify )
        nerrors1 = verify( fasta1, fasta2,
                           options.verify_num_iterations,
                           options.verify_fragment_size,
                           stdout=options.stdout )
        options.stdout.write("errors=%i\n" % (nerrors1) )        
        nerrors2 = IndexedFasta.verify( fasta2, fasta1,
                                        options.verify_num_iterations,
                                        options.verify_fragment_size,
                                        stdout=options.stdout )
        options.stdout.write("errors=%i\n" % (nerrors2) )        
    elif options.compress_index:
        fasta = IndexedFasta.IndexedFasta( args[0] ) 
        fasta.compressIndex()
    else:
        if options.loglevel >= 1:
            options.stdlog.write("# creating database %s\n" % args[0])            
            options.stdlog.write("# indexing the following files: \n# %s\n" %\
                                 (" \n# ".join( args[1:] ) ))
            options.stdlog.flush()

            if synonyms:
                options.stdlog.write("# Applying the following synonyms:\n" )
                for k,v in synonyms.items():
                    options.stdlog.write( "# %s=%s\n" % (k, ",".join(v) ) )
                options.stdlog.flush()
        if len(args) < 2:
            print globals()["__doc__"]
            sys.exit(1)
            
        iterator = IndexedFasta.MultipleFastaIterator( args[1:],     
                                                       regex_identifier = options.regex_identifier,
                                                       format = options.file_format )

        IndexedFasta.createDatabase( args[0], 
                                     iterator, 
                                     synonyms = synonyms,
                                     random_access_points = options.random_access_points,
                                     compression = options.compression,
                                     clean_sequence = options.clean_sequence,
                                     allow_duplicates = options.allow_duplicates,
                                     translator = options.translator,
                                     force = options.force )

    
    E.Stop()

if __name__ == "__main__": 
    sys.exit( main() )




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
MACS.py - Parser for MACS output
================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

The :mod:`Pipeline` module contains various utility functions
for parsing MACS output.

API
----

'''

import collections

MacsPeak = collections.namedtuple( "MacsPeak", "contig start end length summit tags pvalue fold fdr" )

def iterateMacsPeaks( infile ):
    '''iterate over peaks.xls file and return parsed data.
    The fdr is converted from percent to values between 0 and 1.
    '''
 
    for line in infile:
        if line.startswith("#"): continue
        if line.startswith("chr\tstart"): continue
        # skip empty lines
        if line.startswith("\n"): continue

        data = line[:-1].split("\t")

        if len(data) == 9:
            # convert % fdr 
            data[8] = float(data[8]) / 100.0
        elif len(data) == 8:
            ## if no fdr given, set to 0
            # data.append( 0.0 )
            ## Steve - I don't understand this so I'm commenting it out and raising an error
            raise ValueError( "FDR value not set line %s" % line )
        else:
            raise ValueError( "could not parse line %s" % line )

        # these are 1-based coordinates
        # macs can have negative start coordinates
        # start
        data[1] = max(int(data[1]) - 1, 0)
        # end
        data[2] = int(data[2])
        # length
        data[3] = int(data[3])
        # summit
        data[4] = int(data[4])
        # ntags
        data[5] = int(data[5])
        # -10log10(pvalue)
        data[6] = float(data[6])
        # fold
        data[7] = float(data[7])
        
        yield MacsPeak._make( data )

Macs2Peak = collections.namedtuple( "Macs2Peak", "contig start end length summit pileup pvalue fold fdr name" )

def iterateMacs2Peaks( infile ):
    '''iterate over peaks.xls file and return parsed data.
    The fdr is converted from percent to values between 0 and 1.
    '''
 
    for line in infile:
        if line.startswith("#"): continue
        if line.startswith("chr\tstart"): continue
        # skip empty lines
        if line.startswith("\n"): continue

        data = line[:-1].split("\t")

        if len(data) == 10:
            # convert % fdr 
            data[8] = 10**-float(data[8])
        elif len(data) == 8:
            # if no fdr given, set to 0
            #data.append( 0.0 )
            raise ValueError( "No FDR found line %s" % line )
        else:
            raise ValueError( "could not parse line %s" % line )

        # these are 1-based coordinates
        # macs can have negative start coordinates
        # start
        data[1] = max(int(data[1]) - 1, 0)
        # end
        data[2] = int(data[2])
        # length
        data[3] = int(data[3])
        # summit
        data[4] = int(data[4])
        # ntags
        data[5] = float(data[5])
        # -10log10(pvalue)
        data[6] = float(data[6])
        # fold
        data[7] = float(data[7])
        
        yield Macs2Peak._make( data )




################################################################################
#   Gene prediction pipeline 
#
#   $Id: trees2tree.py 2782 2009-09-10 11:40:29Z andreas $
#
#   Copyright (C) 2004 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
trees2tree.py - aggregate trees into a single tree
==================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script reads a collection of trees from stdin and aggregates
them into fewer or a single tree.

Possible aggregation functions are:
   * min, max, sum, mean: min, max, sum, mean of branch-lengths
   * nr: write a set of non-redundant trees.

Usage
-----

Example::

   python <script_name>.py --help

Type::

   python <script_name>.py --help

for command line help.

Documentation
-------------

Code
----

""" 
import os
import sys
import string
import re
import optparse
import math

import scipy
import scipy.stats
import CGAT.Experiment as E
import CGAT.TreeTools as TreeTools
import CGAT.IOTools as IOTools
import CGAT.WrapperPhylip as WrapperPhylip

def getBestTree( trees, method = "select-largest"):
    """select best tree out of a set of trees."""

    if method == "select-largest":
        sizes = zip( map( lambda x: len(x.get_taxa()), trees ), range(len(trees)))
        sizes.sort()

        best_tree = sizes[-1][1]
        if options.loglevel >= 3:
            for x in range(len(trees)):
                if x == best_tree: continue
                options.stdlog.write("# skipped tree: %s: %s\n" % (trees[x].name, TreeTools.Tree2Newick( trees[x])))
        
        return trees[best_tree]
    
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: trees2tree.py 2782 2009-09-10 11:40:29Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-m", "--method", dest="method", type="choice",
                      choices=("counts", "min", "max", "sum", "mean", "median", "stddev", "non-redundant", "consensus",
                               "select-largest"), 
                      help="aggregation function."  )

    parser.add_option("-r", "--regex-id", dest="regex_id", type="string",
                      help="regex pattern to extract identifier from tree name for the selection functions." )

    parser.add_option("-w", "--write-values", dest="write_values", type="string",
                      help="if processing multiple trees, write values to file." )

    parser.add_option("-e", "--error-branchlength", dest="error_branchlength", type="float",
                      help="set branch length without counts to this value." )

    parser.set_defaults(
        method = "mean",
        regex_id = None,
        filtered_branch_lengths = (-999.0,999.0),
        write_values = None,
        error_branchlength = None,
        separator=":",
        )

    (options, args) = E.Start( parser, add_pipe_options = True )

    if options.loglevel >= 2:
        options.stdlog.write( "# reading trees from stdin.\n" )
        options.stdlog.flush()

    nexus = TreeTools.Newick2Nexus( sys.stdin )
    if options.loglevel >= 1:
        options.stdlog.write( "# read %i trees from stdin.\n" % len(nexus.trees))

    nskipped = 0
    ninput = len(nexus.trees)
    noutput = 0
    nerrors = 0

    if options.method == "non-redundant":
        ## compute non-redudant trees
        template_trees = []
        template_counts = []
        ntree = 0
        for tree in nexus.trees:

            for x in range(0, len(template_trees)):
                is_compatible, reason = TreeTools.IsCompatible( tree, template_trees[x] )
                if is_compatible:
                    template_counts[x] += 1
                    break
            else:
                template_counts.append( 1 )
                template_trees.append( tree )

            if options.loglevel >= 2:
                options.stdlog.write("# tree=%i, ntemplates=%i\n" % (ntree, len(template_trees)))
                
            ntree += 1
            
        for x in range(0, len(template_trees)):
            if options.loglevel >= 1:
                options.stdlog.write("# tree: %i, counts: %i, percent=%5.2f\n" %\
                                     (x, template_counts[x], template_counts[x] * 100.0 / ntotal))
            options.stdout.write(TreeTools.Tree2Newick( template_trees[x] ) + "\n")

    elif options.method in ("select-largest",):
        ## select one of the trees with the same name.
        clusters = {}
        for x in range(0, len(nexus.trees)):
            n = nexus.trees[x].name

            if options.regex_id:
                n = re.search(options.regex_id, n).groups()[0]
                
            if n not in clusters: clusters[n] = []
            clusters[n].append(x)

        new_trees = []
        
        for name, cluster in clusters.items():
            new_trees.append( getBestTree( [ nexus.trees[x] for x in cluster], options.method ) )
            
        for x in range(0, len(new_trees)):
            options.stdout.write(">%s\n" % new_trees[x].name )
            options.stdout.write(TreeTools.Tree2Newick( new_trees[x],  ) + "\n")
            noutput += 1

        nskipped = ntotal - noutput
            
    elif options.method == "consensus":
        
        phylip = WrapperPhylip.Phylip()
        phylip.setLogLevel( options.loglevel - 2 )
        phylip.setProgram( "consense" )
        phylip_options = []
        phylip_options.append("Y")
        
        phylip.setOptions( phylip_options )
        phylip.setTrees( nexus.trees )

        result = phylip.run()

        options.stdout.write("# consensus tree built from %i trees\n" % (phylip.mNInputTrees) )
        options.stdout.write(TreeTools.Tree2Newick( result.mNexus.trees[0] ) + "\n")
        noutput = 1

    else:
        if options.method in ("min", "max", "sum", "mean", "counts"):

            xtree = nexus.trees[0]
            for n in xtree.chain.keys():
                if xtree.node(n).data.branchlength in options.filtered_branch_lengths:
                    xtree.node(n).data.branchlength = 0
                ntotals = [1] * len(xtree.chain.keys())
                    
            if options.method == "min":
                f = min
            elif options.method == "max":
                f = max
            elif options.method == "sum":
                f = lambda x, y: x + y
            elif options.method == "mean":
                f = lambda x, y: x + y
            elif options.method == "counts":
                f = lambda x, y: x + 1
                for n in xtree.chain.keys():
                    if xtree.node(n).data.branchlength not in options.filtered_branch_lengths:
                        xtree.node(n).data.branchlength = 1
                    else:
                        xtree.node(n).data.branchlength = 0                        
            else:
                raise "unknown option %s" % options.method
            
            for tree in nexus.trees[1:]:
                
                for n in tree.chain.keys():
                    if tree.node(n).data.branchlength not in options.filtered_branch_lengths:
                        xtree.node(n).data.branchlength = f(xtree.node(n).data.branchlength, tree.node(n).data.branchlength)
                        ntotals[n] += 1

            if options.method == "mean":
                for n in xtree.chain.keys():
                    if ntotals[n] > 0:
                        xtree.node(n).data.branchlength = float(xtree.node(n).data.branchlength) / ntotals[n]
                    else:
                        if options.error_branchlength != None:
                            xtree.node(n).data.branchlength = options.error_branchlength
                            if options.loglevel >= 1:
                                options.stdlog.write("# no counts for node %i - set to %f\n" % (n, options.error_branchlength) )
                                nerrors += 1
                        else:
                            raise "no counts for node %i" % n
                        
        else:
            ## collect all values for trees
            values = [ [] for x in range(TreeTools.GetSize( nexus.trees[0] )) ]
            
            for tree in nexus.trees:
                for n, node in tree.chain.items():
                    if node.data.branchlength not in options.filtered_branch_lengths:
                        values[n].append( node.data.branchlength )

            tree = nexus.trees[0]
            for n, node in tree.chain.items():
                if len(values[n]) > 0:
                    if options.method == "stddev":
                        node.data.branchlength = scipy.std( values[n] )
                    elif options.method == "median":
                        node.data.branchlength = scipy.median( values[n] )
                else:
                    if options.error_branchlength != None:
                        node.data.branchlength = options.error_branchlength
                        if options.loglevel >= 1:
                            options.stdlog.write("# no counts for node %i - set to %f\n" % (n, options.error_branchlength) )
                            nerrors += 1
                    else:
                        raise "no counts for node %i" % n
                    
            if options.write_values:
                outfile = open(options.write_values, "w" )
                for n, node in tree.chain.items():
                    values[n].sort()
                    id = options.separator.join( sorted(TreeTools.GetLeaves( tree, n )) )
                    outfile.write( "%s\t%s\n" % (id, ";".join( map(str, values[n] ) ) ))
                outfile.close()
                
        del nexus.trees[1:]
        options.stdout.write(TreeTools.Nexus2Newick( nexus ) + "\n")
        noutput = 1
        
    if options.loglevel >= 1:
        options.stdlog.write( "# ntotal=%i, nskipped=%i, noutput=%i, nerrors=%i\n" % (ninput, nskipped, noutput, nerrors))
        
    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
gff2stats.py - count features, etc. in gff file
===============================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script computes the number of intervals per feature,
source, gene_id and transcript_id in one or more :term:`gff` 
or :term:`gtf` formatted files.

For a gff file, the implemented counters are:

1. number of intervals per contig, strand, feature and source

For a gtf file, the additional implemented counters are:

1. number of genes, transcripts, single exon transcripts
2. summary statistics for exon numbers, exon sizes, intron sizes and transcript sizes

The output is a tab-separated table.

Usage
-----

Example::

   python gff2stats.py --help

Type::

   python gff2stats.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import string
import re
import optparse
import collections
import CGAT.Experiment as E
import CGAT.GTF as GTF
import CGAT.Stats as Stats
import CGAT.IOTools as IOTools
import CGAT.Intervals as Intervals

class counter_gff:

    fields = ("contigs", "strands", "features", "sources")

    def __init__(self, iter ):
        self.iter = iter

        self.counts_contigs = collections.defaultdict( int )
        self.counts_strands = collections.defaultdict( int )
        self.counts_features = collections.defaultdict( int )
        self.counts_sources = collections.defaultdict( int )
        
    def next(self):
        
        entry = self.iter.next()
        
        self.counts_contigs[entry.contig] += 1
        self.counts_features[entry.feature] += 1
        self.counts_sources[entry.source] += 1
        self.counts_strands[entry.strand] += 1
    
        return entry 

    def __iter__(self):
        return self

    def __str__(self):
        return "\t".join( map(str, ( len(self.counts_contigs),
                                     len(self.counts_strands),
                                     len(self.counts_features),
                                     len(self.counts_sources) )))

class counter_exons:
    
    fields = ( "genes", "transcripts", "single_exon_transcripts", ) +\
        tuple([ "exon_count_%s" % x for x in Stats.Summary.fields ] ) +\
        tuple([ "exon_size_%s" % x for x in Stats.Summary.fields ] ) +\
        tuple([ "intron_size_%s" % x for x in Stats.Summary.fields ] ) +\
        tuple([ "transcript_size_%s" % x for x in Stats.Summary.fields ] )
    
    def __init__(self, iter ):

        self.iter = iter

        self.counts_gene_ids = collections.defaultdict( int )
        self.counts_transcript_ids = collections.defaultdict( int )
        self.counts_exons_per_transcript = collections.defaultdict( list )

    def next(self ):

        while 1:
            entry = self.iter.next()
            if entry.feature == "exon": break

        self.counts_gene_ids[entry.gene_id] += 1
        self.counts_transcript_ids[entry.transcript_id] += 1
        self.counts_exons_per_transcript[entry.transcript_id].append( (entry.start, entry.end) )

        return entry

    def __iter__(self):
        return self
    
    def __str__(self):
        
        single_exon_transcripts = 0
        exons_per_transcript = []
        intron_sizes = []
        transcript_lengths = []
        exon_sizes = []
        
        for x in self.counts_exons_per_transcript.values():

            x.sort()
            x = Intervals.combine( x )
            transcript_lengths.append ( x[-1][1] - x[0][0] )

            exons_per_transcript.append( len(x))
        
            for start, end in x:
                exon_sizes.append( end - start )
    
            if len(x) == 1: 
                single_exon_transcripts += 1
                continue

            last_end = x[0][1]
            for start, end in x[1:]:
                intron_sizes.append( start - last_end )
                last_end = end

        return "\t".join( map(str, ( len(self.counts_gene_ids),
                                     len(self.counts_transcript_ids),
                                     single_exon_transcripts,
                                     Stats.Summary(exons_per_transcript),
                                     Stats.Summary(exon_sizes),
                                     Stats.Summary(intron_sizes),
                                     Stats.Summary(transcript_lengths),
                                     )) )
    
##------------------------------------------------------------------------
def main( argv = sys.argv ):

    parser = E.OptionParser( version = "%prog version: $Id: gff2stats.py 2781 2009-09-10 11:33:14Z andreas $",
                                    usage=globals()["__doc__"])

    parser.add_option("--is-gtf", dest="is_gtf", action="store_true",
                      help="input is gtf.")

    parser.set_defaults(
        is_gtf = False,
        )

    (options, args) = E.Start( parser, add_output_options = True )

    if len(args) == 0:
        files = [ sys.stdin ]
    else: 
        files = args
    
    options.stdout.write( "track\t%s" % ("\t".join(counter_gff.fields) ))
        
    if options.is_gtf:
        options.stdout.write( "\t%s" % ("\t".join(counter_exons.fields)))
    options.stdout.write("\n")

    for f in files:
        if f == options.stdin:
            infile = f
            options.stdout.write( "stdin" )
        else:
            infile = IOTools.openFile( f )
            options.stdout.write( f )

        counters = []
        if options.is_gtf:
            iterator = GTF.iterator( infile )
            counters.append( counter_gff( iterator ) )
            counters.append( counter_exons( counters[0] ) )
        else:
            iterator = GTF.iterator( infile )
            counters.append( counter_gff( iterator ) )

        c = counters[-1]
        for x in c: pass

        for c in counters:
            options.stdout.write("\t%s" % str( c ))
        options.stdout.write( "\n" )        
        

        if infile != sys.stdin: infile.close()

    E.Stop()

if __name__=="__main__":
    sys.exit( main( sys.argv ) )
    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
sequences2mali.py - build pileup mali from a set of sequences
=============================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

convert a set of sequences into a multiple alignment

Usage
-----

Example::

   python sequences2mali.py --help

Type::

   python sequences2mali.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse
import math
import time

import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import CGAT.Mali as Mali
import alignlib
import CGAT.FastaIterator as FastaIterator

def convertMali2Mali( mali ):
    """convert a mali to a profile."""

    new_mali = alignlib.makeMultipleAlignment()
    for id in mali.getIdentifiers():
        s = alignlib.makeAlignatumFromString( mali[id] )
        s.thisown = 0
        new_mali.addAlignatum( s )

    return new_mali

##------------------------------------------------------------
if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: sequences2mali.py 2782 2009-09-10 11:40:29Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-i", "--input-format", dest="input_format", type="choice",
                      choices=("plain", "fasta", "clustal", "stockholm", "phylip" ),
                      help="input format of multiple alignment"  )
    
    parser.add_option("-o", "--output-format", dest="output_format", type="choice",
                      choices=("plain", "fasta", "stockholm", "phylip"),
                      help="output format of multiple alignment"  )

    parser.add_option("-m", "--method", dest="method", type="choice",
                      choices=("add",),
                      help="""method to use to build multiple alignment.""")

    parser.add_option("-p", "--parameters", dest="parameters", type="string",
                      help="parameter stack for methods that require one."  )

    parser.add_option("-a", "--alignment-method", dest="alignment_method", type="choice",
                      choices=("sw", "nw"),
                      help="alignment_method [%default]."  )


    parser.set_defaults(
        input_format="fasta",
        output_format="fasta",
        method = None,
        parameters = "",
        gop = -10.0,
        gep = -1.0,
        alignment_method = "sw",
        )

    (options, args) = E.Start( parser )

    options.parameters = options.parameters.split(",")    

    iterator = FastaIterator.iterate( sys.stdin )

    if options.method == "add":
        
        mali = Mali.Mali()

        mali.readFromFile( open(options.parameters[0], "r"), format = options.input_format )
        del options.parameters[0]

        old_length = mali.getLength()
        
        new_mali = convertMali2Mali( mali )

        if options.alignment_method == "sw":
            alignator = alignlib.makeAlignatorFullDP( options.gop, options.gep )
        else:
            alignator = alignlib.makeAlignatorFullDPGlobal( options.gop, options.gep )            
        
        while 1:
            cur_record = iterator.next()
            if cur_record is None: break

            map_mali2seq = alignlib.makeAlignataVector()

            sequence = alignlib.makeSequence( cur_record.sequence )
            profile = alignlib.makeProfileFromMali( new_mali )

            if options.loglevel >= 4:
                options.stdlog.write(profile.Write())

            alignator.Align( profile, sequence, map_mali2seq )

            if options.loglevel >= 3:
                options.stdlog.write( map_mali2seq.Write() )

            ## add sequence to mali
            a = alignlib.makeAlignatumFromString( cur_record.sequence )
            a.thisown = 0
                
            new_mali.addAlignatum( a, map_mali2seq, 1, 1, 1, 1, 1 )

            id = cur_record.title
            mali.mIdentifiers.append( id )
            mali.mMali[id] = Mali.AlignedString( id, 0, len(cur_record.sequence), new_mali.getRow( new_mali.getWidth() - 1 ).getString() )

        # substitute 
        for x in range(old_length):
            mali.mMali[mali.mIdentifiers[x]].mString = new_mali.getRow( x ).getString()
            
        mali.writeToFile( sys.stdout, format = options.output_format )
        
    E.Stop()
    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
malis2mali.py - concatenate multiple alignments
===============================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

concatenate multiple alignments into a single multiple alignment. 

This script requires the following input:

1. Multiple alignments 

The multiple alignments can be given as separate files (option --pattern-mali contains
a '%%s') or in a single file.

2. A list of components

This file maps sequence identifiers in the multiple alignment file(s)
to multiple alignment identifiers. The format for this file is a tab separated table 
containing the following fields::

    1       sequence        a sequence identifier
    2       input_id        The id under which the alignment is found. This value is substituted
                            for the '%%s' in --pattern-mali.
    3       component_id    The component_id or multiple alignment identifier.
                            This field is optional. If no third column is specified, the input_id
                            is used.

Methods:

filter-variants
   only keep variant columns in the multiple alignment. Set ``--width`` to 3 to test for variant
   codons. Gaps will be ignored.

Usage
-----

Example::

   python malis2mali.py --help

Type::

   python malis2mali.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse
import math
import time
import random
import types


import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import CGAT.Mali as Mali

## import shared helper functions from malis2malis
from malis2malis import *

##------------------------------------------------------------
if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: malis2mali.py 2782 2009-09-10 11:40:29Z andreas $", usage = globals()["__doc__"])

    addOptions( parser )

    parser.add_option( "--filename-coordinates", dest="filename_coordinates", type="string",
                      help="filename of coordinates that constitute the multiple alignment."  )

    parser.add_option( "--filename-identifiers", dest="filename_identifiers", type="string",
                      help="filename with list of identifiers to use."  )

    parser.add_option("-x", "--pattern-identifier", dest="pattern_identifier", type="string",
                      help="pattern to extract identifier from a sequence header."  )

    parser.add_option("-w", "--width", dest="width", type="int",
                      help="width of an alignment column (choose 3 for codon alignments) [default=%default]."  )

    parser.add_option("-m", "--method", dest="methods", type="choice",
                      choices = ("filter-variants", ),
                      help = "methods to apply" )

    parser.add_option("-p", "--parameters", dest="parameters", type="string",
                      help="parameter stack for methods that require one."  )

    parser.add_option("--mask-acgtn", dest="mask_actgn", action="store_true",
                      help="mask. Anything not [ACGTN] will be N."  )

    parser.set_defaults(
        pattern_identifier="(^\S+)",
        methods = [],
        parameters = "",
        filename_identifiers = None,
        filename_coordinates = None,
        mask_acgtn = False,
        )

    (options, args) = E.Start( parser )

    options.parameters = options.parameters.split(",")    

    if not options.pattern_mali:
        raise ValueError("Please specifiy a pattern to find the malis using --pattern-mali")

    ####################################################################
    ####################################################################
    ####################################################################
    ## Read components
    ####################################################################
    map_seq_id2component, map_component2seq_id, map_component2input_id = \
        readComponents( options )

    ####################################################################
    ####################################################################
    ####################################################################
    ## Read regions to mask
    ####################################################################
    map_component2masks = readMasks( options, map_component2input_id )

    ####################################################################
    ####################################################################
    ####################################################################
    ## Read regions to extract
    ####################################################################
    map_component2extracts = readExtracts( options, map_component2input_id )

    ####################################################################
    ####################################################################
    ####################################################################
    ## read identifiers
    ####################################################################
    if options.filename_identifiers:
        identifiers, nerrors = IOTools.ReadList( open(options.filename_identifiers, "r") )
        identifiers_set = set(identifiers)
    else:
        identifiers = None
        identifiers_set = None
        
    ####################################################################
    ####################################################################
    ####################################################################
    ## Prepare for run
    ####################################################################

    rx_identifier = re.compile(options.pattern_identifier)

    ## build list of concatenated malis
    sequences = {}
    if identifiers:
        for id in identifiers_set:
            sequences[id] = []
    else:
        identifiers_set = set()
        for seq_id in map_seq_id2component.keys():
            id = rx_identifier.search(seq_id).groups()[0]
            sequences[id] = []
            identifiers_set.add( id )
        identifiers = list(identifiers_set)
        identifiers.sort()
    
    component_ids = map_component2seq_id.keys()
    component_ids.sort()

    if options.test:
        component_ids = component_ids[:options.test]
        
    ####################################################################
    ####################################################################
    ####################################################################
    ## Build list of components to output.
    ####################################################################
    component_ids, map_sample2reference = selectComponents( component_ids, 
                                                            map_component2seq_id,
                                                            map_component2input_id,
                                                            None,
                                                            options )

    nskipped = 0
    new_component_ids = []

    for component_id in component_ids:

        try:
            mali = getMali( component_id, 
                            map_component2seq_id, 
                            map_component2input_id,
                            None,
                            options )
        except OSError, msg:
            E.warn("could not find mali %s: %s" % (component_id, msg))
            nskipped += 1
            continue

        ###############################################################
        ###############################################################
        ###############################################################
        ## check if all identifiers in component are present in mali
        ## and build a temporary alignment with all of those found
        component_set = set(map_component2seq_id[component_id])
        if len(component_set.difference( set(mali.getIdentifiers()))) != 0:
            nskipped += 1
            continue

        found = {}
        is_double = None
        temp_mali = Mali.Mali()
        temp_mali.setName( str(component_id) )

        for seq_id in map_component2seq_id[component_id]:
            id = rx_identifier.search(seq_id).groups()[0]
            if id not in identifiers_set:
                continue
            if id in found:

                if options.skip_doubles:
                    if options.loglevel >= 1:
                        options.stdlog.write("# component %s: removed double entry %s\n" % (component_id, seq_id ))
                    continue
                else:
                    is_double = id
                    break
                
            if options.output_format == "codeml":
                if len(mali[seq_id]) % 3 != 0:
                    raise "length of sequence %s is not a multiple of 3: %i" % (seq_id, len(mali[seq_id]))

            ## change identifier to id
            found[id] = True
            entry = mali.getEntry( seq_id ) 
            temp_mali.addSequence( id, entry.mFrom, entry.mTo, entry.mString )

        if is_double:
            nskipped += 1
            if options.loglevel >= 1:
                options.stdlog.write("# component %s: skipped because it contains double entry %s\n" % (component_id, is_double ))
            continue
        
        if set(found.keys()) != identifiers_set:
            nskipped += 1
            if options.loglevel >= 1:
                options.stdlog.write("# component %s: skipped because incomplete: %s\n" % (component_id, str(found.keys())))
            continue

        ###############################################################
        ###############################################################
        ###############################################################
        ## mask the temporary alignment
        maskAlignment( temp_mali, 
                       map_component2masks, 
                       map_component2extracts, 
                       map_sample2reference,
                       options )
        
        for id, o in temp_mali.items():
            if options.mask_acgtn:
                s = re.sub( "[^ACGTNactgn]", "N", o.mString)
            else:
                s = o.mString
            sequences[id].append( s )

        new_component_ids.append(component_id)

        ## if we only sample, stop if you have reached
        ## the desired number
        if options.sample and len(new_component_ids) == options.sample:
            break
        
    component_ids = new_component_ids
    
    nnucleotides = reduce( lambda x, y: x + y, map(len, sequences[identifiers[0]] ) )
    nseqs = len(sequences)
    ngenes = len(sequences)

    if options.output_format == "fasta":
        
        for id in identifiers:
            options.stdout.write( ">%s\n" % id )
            for seq in sequences[id]:
                options.stdout.write( "%s\n" % seq )                
            
    elif options.output_format == "codeml":

        ## Codeml output with G option. The format is
        ## "nseqs nnucleotides"
        ## "G ngenes genelengths in codons"

        options.stdout.write( "%i %i G\n" % (nseqs, nnucleotides))
        options.stdout.write( "G %i %s\n" % (ngenes, " ".join(map(lambda x: str(len(x) / 3), sequences[identifiers[0]]) ) ) )

        for id in identifiers:
            options.stdout.write( "%s\n" % id )
            for seq in sequences[id]:
                options.stdout.write( "%s\n" % seq )                

    elif options.output_format == "phylip":

        options.stdout.write( "%i %i\n" % (nseqs, nnucleotides))

        for id in identifiers:
            options.stdout.write( "%s\n" % id )
            for seq in sequences[id]:
                options.stdout.write( "%s\n" % seq )                
                
    if options.filename_coordinates:
            
        outfile = open(options.filename_coordinates, "w")
        outfile.write("component\tlength\tposition\n" )
        p = 0
        lsequences = map(lambda x: len(x), sequences[identifiers[0]])

        if len(lsequences) != len(component_ids):
            ll = []
            for id in identifiers:
                ll.append( len(map(lambda x: len(x), sequences[id]) ) )
            raise "%s: %s, lsequences=%i, ncomponents=%i" % (identifiers[0], str(ll), len(lsequences), len(component_ids))
        
        assert( len(lsequences) == len(component_ids) )
        
        for c in range(len(component_ids)):
            
            outfile.write( "%s\t%i\t%i\n" % (component_ids[c], lsequences[c], p) )
            p += lsequences[c]
            
        outfile.close()

    if options.loglevel >= 1:
        options.stdlog.write("# nsample=%i, nskipped=%i\n" % (len(component_ids), nskipped))

    E.Stop()
    
